@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef __LSQUIC_H__
 #define __LSQUIC_H__
 
@@ -24,8 +24,8 @@ extern "C" {
 #endif
 
 #define LSQUIC_MAJOR_VERSION 2
-#define LSQUIC_MINOR_VERSION 4
-#define LSQUIC_PATCH_VERSION 2
+#define LSQUIC_MINOR_VERSION 12
+#define LSQUIC_PATCH_VERSION 14
 
 /**
  * Engine flags:
@@ -34,9 +34,7 @@ extern "C" {
 /** Server mode */
 #define LSENG_SERVER (1 << 0)
 
-/** Treat stream 3 as headers stream and, in general, behave like the
- *  regular QUIC.
- */
+/** Use HTTP behavior */
 #define LSENG_HTTP  (1 << 1)
 
 #define LSENG_HTTP_SERVER (LSENG_SERVER|LSENG_HTTP)
@@ -47,51 +45,6 @@ extern "C" {
  */
 enum lsquic_version
 {
-
-    /** Q035.  This is the first version to be supported by LSQUIC. */
-    /* Support for this version has been removed.  The comment remains to
-     * document the changes.
-     */
-
-    /*
-     * Q037.  This version is like Q035, except the way packet hashes are
-     * generated is different for clients and servers.  In addition, new
-     * option NSTP (no STOP_WAITING frames) is rumored to be supported at
-     * some point in the future.
-     */
-    /* Support for this version has been removed.  The comment remains to
-     * document the changes.
-     */
-
-    /*
-     * Q038.  Based on Q037, supports PADDING frames in the middle of packet
-     * and NSTP (no STOP_WAITING frames) option.
-     */
-    /* Support for this version has been removed.  The comment remains to
-     * document the changes.
-     */
-
-    /**
-     * Q039.  Switch to big endian.  Do not ack acks.  Send connection level
-     * WINDOW_UPDATE frame every 20 sent packets which do not contain
-     * retransmittable frames.
-     */
-    LSQVER_039,
-
-    /*
-     * Q041.  RST_STREAM, ACK and STREAM frames match IETF format.
-     */
-    /* Support for this version has been removed.  The comment remains to
-     * document the changes.
-     */
-
-    /*
-     * Q042.  Receiving overlapping stream data is allowed.
-     */
-    /* Support for this version has been removed.  The comment remains to
-     * document the changes.
-     */
-
     /**
      * Q043.  Support for processing PRIORITY frames.  Since this library
      * has supported PRIORITY frames from the beginning, this version is
@@ -99,19 +52,18 @@ enum lsquic_version
      */
     LSQVER_043,
 
-    /**
-     * Q044.  IETF-like packet headers are used.  Frames are the same as
-     * in Q043.  Server never includes CIDs in short packets.
-     */
-    /* Support for this version has been removed.  The comment remains to
-     * document the changes.
-     */
-
     /**
      * Q046.  Use IETF Draft-17 compatible packet headers.
      */
     LSQVER_046,
 
+    /**
+     * Q050.  Variable-length QUIC server connection IDs.  Use CRYPTO frames
+     * for handshake.  IETF header format matching invariants-06.  Packet
+     * number encryption.  Initial packets are obfuscated.
+     */
+    LSQVER_050,
+
 #if LSQUIC_USE_Q098
     /**
      * Q098.  This is a made-up, experimental version used to test version
@@ -125,9 +77,14 @@ enum lsquic_version
 #endif
 
     /**
-     * IETF QUIC Draft-23
+     * IETF QUIC Draft-25
      */
-    LSQVER_ID23,
+    LSQVER_ID25,
+
+    /**
+     * IETF QUIC Draft-27
+     */
+    LSQVER_ID27,
 
     /**
      * Special version to trigger version negotiation.
@@ -139,7 +96,7 @@ enum lsquic_version
 };
 
 /**
- * We currently support versions 39, 43, 46, and IETF Draft-23
+ * We currently support versions 43, 46, 50, Draft-25, and Draft-27.
  * @see lsquic_version
  */
 #define LSQUIC_SUPPORTED_VERSIONS ((1 << N_LSQVER) - 1)
@@ -147,18 +104,20 @@ enum lsquic_version
 /**
  * List of versions in which the server never includes CID in short packets.
  */
-#define LSQUIC_FORCED_TCID0_VERSIONS (1 << LSQVER_046)
+#define LSQUIC_FORCED_TCID0_VERSIONS ((1 << LSQVER_046)|(1 << LSQVER_050))
 
 #define LSQUIC_EXPERIMENTAL_VERSIONS ( \
                             (1 << LSQVER_VERNEG) | LSQUIC_EXPERIMENTAL_Q098)
 
 #define LSQUIC_DEPRECATED_VERSIONS 0
 
-#define LSQUIC_GQUIC_HEADER_VERSIONS ((1 << LSQVER_039) | (1 << LSQVER_043))
+#define LSQUIC_GQUIC_HEADER_VERSIONS (1 << LSQVER_043)
 
-#define LSQUIC_IETF_VERSIONS ((1 << LSQVER_ID23) | (1 << LSQVER_VERNEG))
+#define LSQUIC_IETF_VERSIONS ((1 << LSQVER_ID25) | (1 << LSQVER_ID27) \
+                                                    | (1 << LSQVER_VERNEG))
 
-#define LSQUIC_IETF_DRAFT_VERSIONS ((1 << LSQVER_ID23) | (1 << LSQVER_VERNEG))
+#define LSQUIC_IETF_DRAFT_VERSIONS ((1 << LSQVER_ID25) | (1 << LSQVER_ID27) \
+                                                    | (1 << LSQVER_VERNEG))
 
 enum lsquic_hsk_status
 {
@@ -183,7 +142,7 @@ enum lsquic_hsk_status
 
 /**
  * @struct lsquic_stream_if
- * @brief The definition of callback functions call by lsquic_stream to
+ * @brief The definitions of callback functions called by lsquic_stream to
  * process events.
  *
  */
@@ -225,7 +184,7 @@ struct lsquic_stream_if {
      */
     void (*on_hsk_done)(lsquic_conn_t *c, enum lsquic_hsk_status s);
     /**
-     * When server sends a token in NEW_TOKEN frame, this callback is called.
+     * When client receives a token in NEW_TOKEN frame, this callback is called.
      * The callback is optional.
      */
     void (*on_new_token)(lsquic_conn_t *c, const unsigned char *token,
@@ -246,6 +205,9 @@ struct ssl_st;
  * constructor.
  */
 
+/* `sni' may be NULL if engine is not HTTP mode and client TLS transport
+ * parameters did not include the SNI.
+ */
 typedef struct ssl_ctx_st * (*lsquic_lookup_cert_f)(
     void *lsquic_cert_lookup_ctx, const struct sockaddr *local, const char *sni);
 
@@ -316,8 +278,6 @@ typedef struct ssl_ctx_st * (*lsquic_lookup_cert_f)(
 
 #define LSQUIC_DF_STTL               86400
 #define LSQUIC_DF_MAX_INCHOATE     (1 * 1000 * 1000)
-#define LSQUIC_DF_SUPPORT_SREJ_SERVER  1
-#define LSQUIC_DF_SUPPORT_SREJ_CLIENT  0       /* TODO: client support */
 /** Do not use NSTP by default */
 #define LSQUIC_DF_SUPPORT_NSTP     0
 /** TODO: IETF QUIC clients do not support push */
@@ -364,8 +324,23 @@ typedef struct ssl_ctx_st * (*lsquic_lookup_cert_f)(
 /** Allow migration by default */
 #define LSQUIC_DF_ALLOW_MIGRATION 1
 
+/** Use QL loss bits by default */
+#define LSQUIC_DF_QL_BITS 2
+
+/** Turn spin bit on by default */
+#define LSQUIC_DF_SPIN 1
+
+/** Turn off delayed ACKs extension by default */
+#define LSQUIC_DF_DELAYED_ACKS 0
+
+/** Turn on timestamp extension by default */
+#define LSQUIC_DF_TIMESTAMPS 1
+
 /* 1: Cubic; 2: BBR */
-#define LSQUIC_DF_CC_ALGO 2
+#define LSQUIC_DF_CC_ALGO 1
+
+/** By default, incoming packet size is not limited. */
+#define LSQUIC_DF_MAX_PACKET_SIZE_RX 0
 
 struct lsquic_engine_settings {
     /**
@@ -460,13 +435,6 @@ struct lsquic_engine_settings {
      */
     unsigned        es_max_inchoate;
 
-    /**
-     * Support SREJ: for client side, this means supporting server's SREJ
-     * responses (this does not work yet) and for server side, this means
-     * generating SREJ instead of REJ when appropriate.
-     */
-    int             es_support_srej;
-
     /**
      * Setting this value to 0 means that
      *
@@ -667,7 +635,7 @@ struct lsquic_engine_settings {
 
     /**
      * Source Connection ID length.  Only applicable to the IETF QUIC
-     * versions.  Valid values are 4 through 18, inclusive.
+     * versions.  Valid values are 0 through 20, inclusive.
      *
      * Default value is @ref LSQUIC_DF_SCID_LEN.
      */
@@ -738,6 +706,51 @@ struct lsquic_engine_settings {
      *  2:  BBR
      */
     unsigned        es_cc_algo;
+
+    /**
+     * Use QL loss bits.  Allowed values are:
+     *  0:  Do not use loss bits
+     *  1:  Allow loss bits
+     *  2:  Allow and send loss bits
+     *
+     * Default value is @ref LSQUIC_DF_QL_BITS
+     */
+    int             es_ql_bits;
+
+    /**
+     * Enable spin bit.  Allowed values are 0 and 1.
+     *
+     * Default value is @ref LSQUIC_DF_SPIN
+     */
+    int             es_spin;
+
+    /**
+     * Enable delayed ACKs extension.  Allowed values are 0 and 1.
+     *
+     * Warning: this is an experimental feature.  Using it will most likely
+     * lead to degraded performance.
+     *
+     * Default value is @ref LSQUIC_DF_DELAYED_ACKS
+     */
+    int             es_delayed_acks;
+
+    /**
+     * Enable timestamps extension.  Allowed values are 0 and 1.
+     *
+     * Default value is @ref LSQUIC_DF_TIMESTAMPS
+     */
+    int             es_timestamps;
+
+    /**
+     * Maximum packet size we are willing to receive.  This is sent to
+     * peer in transport parameters: the library does not enforce this
+     * limit for incoming packets.
+     *
+     * If set to zero, limit is not set.
+     *
+     * Default value is @ref LSQUIC_DF_MAX_PACKET_SIZE_RX
+     */
+    unsigned short  es_max_packet_size_rx;
 };
 
 /* Initialize `settings' to default values */
@@ -780,8 +793,14 @@ struct lsquic_out_spec
  * Returns number of packets successfully sent out or -1 on error.  -1 should
  * only be returned if no packets were sent out.  If -1 is returned or if the
  * return value is smaller than `n_packets_out', this indicates that sending
- * of packets is not possible  No packets will be attempted to be sent out
- * until @ref lsquic_engine_send_unsent_packets() is called.
+ * of packets is not possible.
+ *
+ * If not all packets could be sent out, errno is examined.  If it is not
+ * EAGAIN or EWOULDBLOCK, the connection whose packet cause the error is
+ * closed forthwith.
+ *
+ * No packets will be attempted to be sent out until
+ * @ref lsquic_engine_send_unsent_packets() is called.
  */
 typedef int (*lsquic_packets_out_f)(
     void                          *packets_out_ctx,
@@ -948,14 +967,29 @@ struct lsquic_keylog_if
     void      (*kli_close) (void *handle);
 };
 
-/* TODO: describe this important data structure */
-typedef struct lsquic_engine_api
+/**
+ * This struct contains a list of all callbacks that are used by the engine
+ * to communicate with the user code.  Most of these are optional, while
+ * the following are mandatory:
+ *
+ *  @ref ea_stream_if       The stream interface.
+ *  @ref ea_packets_out     Function to send packets.
+ *  @ref ea_lookup_cert     Function to look up certificates by SNI (used
+ *                            in server mode).
+ *
+ * A pointer to this structure is passed to engine constructor
+ * @ref lsquic_engine_new().
+ */
+struct lsquic_engine_api
 {
     const struct lsquic_engine_settings *ea_settings;   /* Optional */
+    /** Stream interface is required to manage connections and streams. */
     const struct lsquic_stream_if       *ea_stream_if;
     void                                *ea_stream_if_ctx;
+    /** Function to send packets out is required. */
     lsquic_packets_out_f                 ea_packets_out;
     void                                *ea_packets_out_ctx;
+    /** Function to look up certificates by SNI is used in server mode. */
     lsquic_lookup_cert_f                 ea_lookup_cert;
     void                                *ea_cert_lu_ctx;
     struct ssl_ctx_st *                (*ea_get_ssl_ctx)(void *peer_ctx);
@@ -1012,25 +1046,43 @@ typedef struct lsquic_engine_api
      */
     const struct lsquic_keylog_if       *ea_keylog_if;
     void                                *ea_keylog_ctx;
-} lsquic_engine_api_t;
+
+    /**
+     * The optional ALPN string is used by the client @ref LSENG_HTTP
+     * is not set.
+     */
+    const char                          *ea_alpn;
+};
 
 /**
  * Create new engine.
  *
  * @param   lsquic_engine_flags     A bitmask of @ref LSENG_SERVER and
  *                                  @ref LSENG_HTTP
+ *
+ * @param   api                     Required parameter that specifies
+ *                                    various callbacks.
+ *
+ * The engine can be instantiated either in server mode (when LSENG_SERVER
+ * is set) or client mode.  If you need both server and client in your
+ * program, create two engines (or as many as you'd like).
  */
 lsquic_engine_t *
 lsquic_engine_new (unsigned lsquic_engine_flags,
-                   const struct lsquic_engine_api *);
+                   const struct lsquic_engine_api *api);
 
 /**
  * Create a client connection to peer identified by `peer_ctx'.
+ *
+ * To let the engine specify QUIC version, use N_LSQVER.  If zero-rtt info
+ * is supplied, version is picked from there instead.
+ *
  * If `max_packet_size' is set to zero, it is inferred based on `peer_sa':
  * 1350 for IPv6 and 1370 for IPv4.
  */
 lsquic_conn_t *
-lsquic_engine_connect (lsquic_engine_t *, const struct sockaddr *local_sa,
+lsquic_engine_connect (lsquic_engine_t *, enum lsquic_version,
+                       const struct sockaddr *local_sa,
                        const struct sockaddr *peer_sa,
                        void *peer_ctx, lsquic_conn_ctx_t *conn_ctx,
                        const char *hostname, unsigned short max_packet_size,
@@ -1049,8 +1101,8 @@ lsquic_engine_connect (lsquic_engine_t *, const struct sockaddr *local_sa,
  *              This may happen with version negotiation and public reset
  *              packets as well as some packets that may be ignored.
  *
- * @retval -1   Some error occurred.  Possible reasons are invalid packet
- *              size or failure to allocate memory.
+ * @retval -1   An error occurred.  Possible reasons are failure to allocate
+ *              memory and invalid @param sa_local in client mode.
  */
 int
 lsquic_engine_packet_in (lsquic_engine_t *,
@@ -1076,13 +1128,17 @@ lsquic_engine_has_unsent_packets (lsquic_engine_t *engine);
  * Send out as many unsent packets as possibe: until we are out of unsent
  * packets or until @ref ea_packets_out() fails.
  *
- * If @ref ea_packets_out() does fail (that is, it returns an error), this
+ * If @ref ea_packets_out() does fail cannot send all packets, this
  * function must be called to signify that sending of packets is possible
  * again.
  */
 void
 lsquic_engine_send_unsent_packets (lsquic_engine_t *engine);
 
+/**
+ * Destroy engine and all connections and streams in it and free all
+ * memory associated with this engine.
+ */
 void
 lsquic_engine_destroy (lsquic_engine_t *);
 
@@ -1090,6 +1146,18 @@ lsquic_engine_destroy (lsquic_engine_t *);
 unsigned
 lsquic_conn_n_avail_streams (const lsquic_conn_t *);
 
+/**
+ * Create a new request stream.  This causes @ref on_new_stream() callback
+ * to be called.  If creating more requests is not permitted at the moment
+ * (due to number of concurrent streams limit), stream creation is registered
+ * as "pending" and the stream is created later when number of streams dips
+ * under the limit again.  Any number of pending streams can be created.
+ * Use @ref lsquic_conn_n_pending_streams() and
+ * @ref lsquic_conn_cancel_pending_streams() to manage pending streams.
+ *
+ * If connection is going away, @ref on_new_stream() is called with the
+ * stream parameter set to NULL.
+ */
 void
 lsquic_conn_make_stream (lsquic_conn_t *);
 
@@ -1122,14 +1190,41 @@ lsquic_conn_going_away (lsquic_conn_t *);
 void
 lsquic_conn_close (lsquic_conn_t *);
 
-int lsquic_stream_wantread(lsquic_stream_t *s, int is_want);
-ssize_t lsquic_stream_read(lsquic_stream_t *s, void *buf, size_t len);
-ssize_t lsquic_stream_readv(lsquic_stream_t *s, const struct iovec *,
-                                                            int iovcnt);
+/**
+ * Set whether you want to read from stream.  If @param is_want is true,
+ * @ref on_read() will be called when there is readable data in the
+ * stream.  If @param is false, @ref on_read() will not be called.
+ *
+ * Returns previous value of this flag.
+ */
+int
+lsquic_stream_wantread (lsquic_stream_t *s, int is_want);
+
+/**
+ * Read up to @param len bytes from stream into @param buf.  Returns number
+ * of bytes read or -1 on error, in which case errno is set.  Possible
+ * errno values:
+ *
+ *  EBADF           The stream is closed.
+ *  ECONNRESET      The stream has been reset.
+ *  EWOULDBLOCK     There is no data to be read.
+ *
+ * Return value of zero indicates EOF.
+ */
+ssize_t
+lsquic_stream_read (lsquic_stream_t *s, void *buf, size_t len);
+
+/**
+ * Similar to @ref lsquic_stream_read(), but reads data into @param vec.
+ */
+ssize_t
+lsquic_stream_readv (lsquic_stream_t *s, const struct iovec *vec, int iovcnt);
 
 /**
  * This function allows user-supplied callback to read the stream contents.
  * It is meant to be used for zero-copy stream processing.
+ *
+ * Return value and errors are same as in @ref lsquic_stream_read().
  */
 ssize_t
 lsquic_stream_readf (lsquic_stream_t *s,
@@ -1146,15 +1241,31 @@ lsquic_stream_readf (lsquic_stream_t *s,
     size_t (*readf)(void *ctx, const unsigned char *buf, size_t len, int fin),
     void *ctx);
 
-int lsquic_stream_wantwrite(lsquic_stream_t *s, int is_want);
+/**
+ * Set whether you want to write to stream.  If @param is_want is true,
+ * @ref on_write() will be called when it is possible to write data to
+ * the stream.  If @param is false, @ref on_write() will not be called.
+ *
+ * Returns previous value of this flag.
+ */
+int
+lsquic_stream_wantwrite (lsquic_stream_t *s, int is_want);
 
 /**
  * Write `len' bytes to the stream.  Returns number of bytes written, which
  * may be smaller that `len'.
+ *
+ * A negative return value indicates a serious error (the library is likely
+ * to have aborted the connection because of it).
  */
-ssize_t lsquic_stream_write(lsquic_stream_t *s, const void *buf, size_t len);
+ssize_t
+lsquic_stream_write (lsquic_stream_t *s, const void *buf, size_t len);
 
-ssize_t lsquic_stream_writev(lsquic_stream_t *s, const struct iovec *vec, int count);
+/**
+ * Like @ref lsquic_stream_write(), but read data from @param vec.
+ */
+ssize_t
+lsquic_stream_writev (lsquic_stream_t *s, const struct iovec *vec, int count);
 
 /**
  * Used as argument to @ref lsquic_stream_writef()
@@ -1217,13 +1328,18 @@ struct lsquic_http_headers
     lsquic_http_header_t   *headers;
 };
 
-int lsquic_stream_send_headers(lsquic_stream_t *s,
-                               const lsquic_http_headers_t *h, int eos);
+/**
+ * Send headers in @param headers.  This function must be called before
+ * writing to the stream.  The value of @param eos is ignored in IETF QUIC.
+ */
+int
+lsquic_stream_send_headers (lsquic_stream_t *s,
+                               const lsquic_http_headers_t *headers, int eos);
 
 /**
  * Get header set associated with the stream.  The header set is created by
  * @ref hsi_create_header_set() callback.  After this call, the ownership of
- * the header set is trasnferred to the caller.
+ * the header set is transferred to the caller.
  *
  * This call must precede calls to @ref lsquic_stream_read() and
  * @ref lsquic_stream_readv().
@@ -1269,10 +1385,7 @@ int lsquic_stream_close(lsquic_stream_t *s);
 
 /**
  * Get certificate chain returned by the server.  This can be used for
- * server certificate verifiction.
- *
- * If server certificate cannot be verified, the connection can be closed
- * using lsquic_conn_cert_verification_failed().
+ * server certificate verification.
  *
  * The caller releases the stack using sk_X509_free().
  */
@@ -1343,9 +1456,6 @@ int lsquic_stream_set_priority (lsquic_stream_t *s, unsigned priority);
  */
 lsquic_conn_t * lsquic_stream_conn(const lsquic_stream_t *s);
 
-lsquic_stream_t *
-lsquic_conn_get_stream_by_id (lsquic_conn_t *c, lsquic_stream_id_t stream_id);
-
 /** Get connection ID */
 const lsquic_cid_t *
 lsquic_conn_id (const lsquic_conn_t *c);
@@ -1505,17 +1615,18 @@ lsquic_conn_crypto_cipher (const lsquic_conn_t *c);
 enum lsquic_version
 lsquic_str2ver (const char *str, size_t len);
 
+/** Translate ALPN (e.g. "h3", "h3-23", "h3-Q046") to LSQUIC enum */
+enum lsquic_version
+lsquic_alpn2ver (const char *alpn, size_t len);
+
 /**
- * This function closes all mini connections and marks all full connection
+ * This function closes all mini connections and marks all full connections
  * as going away.  In server mode, this also causes the engine to stop
  * creating new connections.
  */
 void
 lsquic_engine_cooldown (lsquic_engine_t *);
 
-struct ssl_st *
-lsquic_hsk_getssl(lsquic_conn_t *conn);
-
 /**
  * Get user-supplied context associated with the connection.
  */
