@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_alarmset.h -- A set of alarms
  */
@@ -27,15 +27,14 @@ enum alarm_id {
     AL_RETX_APP = AL_RETX_INIT + PNS_APP,
     AL_PING,
     AL_IDLE,
-    AL_ACK_INIT,
-    AL_ACK_HSK = AL_ACK_INIT + PNS_HSK,
-    AL_ACK_APP = AL_ACK_INIT + PNS_APP,
+    AL_ACK_APP,
     AL_RET_CIDS,
     AL_CID_THROT,
     AL_PATH_CHAL,
     AL_PATH_CHAL_0 = AL_PATH_CHAL,
     AL_PATH_CHAL_1,
     AL_SESS_TICKET,
+    AL_BLOCKED_KA,      /* Blocked Keep-Alive */
     MAX_LSQUIC_ALARMS
 };
 
@@ -46,16 +45,14 @@ enum alarm_id_bit {
     ALBIT_RETX_HSK  = 1 << AL_RETX_HSK,
     ALBIT_RETX_APP  = 1 << AL_RETX_APP,
     ALBIT_ACK_APP   = 1 << AL_ACK_APP,
-    ALBIT_ACK_INIT  = 1 << AL_ACK_INIT,
-    ALBIT_ACK_HSK   = 1 << AL_ACK_HSK,
     ALBIT_PING      = 1 << AL_PING,
     ALBIT_IDLE      = 1 << AL_IDLE,
     ALBIT_RET_CIDS  = 1 << AL_RET_CIDS,
     ALBIT_CID_THROT = 1 << AL_CID_THROT,
-    ALBIT_PATH_CHAL = 1 << AL_PATH_CHAL,
     ALBIT_PATH_CHAL_0 = 1 << AL_PATH_CHAL_0,
     ALBIT_PATH_CHAL_1 = 1 << AL_PATH_CHAL_1,
     ALBIT_SESS_TICKET = 1 << AL_SESS_TICKET,
+    ALBIT_BLOCKED_KA  = 1 << AL_BLOCKED_KA,
 };
 
 
@@ -94,6 +91,8 @@ void
 lsquic_alarmset_ring_expired (lsquic_alarmset_t *, lsquic_time_t now);
 
 lsquic_time_t
-lsquic_alarmset_mintime (const lsquic_alarmset_t *);
+lsquic_alarmset_mintime (const lsquic_alarmset_t *, enum alarm_id *);
+
+extern const char *const lsquic_alid2str[];
 
 #endif
