@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 // Copyright 2016 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE.chrome file.
@@ -29,9 +29,12 @@
 #include "lsquic_conn_public.h"
 #include "lsquic_util.h"
 #include "lsquic_malo.h"
+#include "lsquic_crand.h"
+#include "lsquic_mm.h"
+#include "lsquic_engine_public.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_BBR
-#define LSQUIC_LOG_CONN_ID lsquic_conn_log_cid(bbr->bbr_conn)
+#define LSQUIC_LOG_CONN_ID lsquic_conn_log_cid(bbr->bbr_conn_pub->lconn)
 #include "lsquic_logger.h"
 
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
@@ -161,14 +164,8 @@ set_startup_values (struct lsquic_bbr *bbr)
 
 
 static void
-lsquic_bbr_init (void *cong_ctl, const struct lsquic_conn_public *conn_pub,
-                                                enum quic_ft_bit retx_frames)
+init_bbr (struct lsquic_bbr *bbr)
 {
-    struct lsquic_bbr *const bbr = cong_ctl;
-    bbr->bbr_conn = conn_pub->lconn;
-    lsquic_bw_sampler_init(&bbr->bbr_bw_sampler, bbr->bbr_conn, retx_frames);
-
-    bbr->bbr_rtt_stats = &conn_pub->rtt_stats;
     bbr->bbr_mode = BBR_MODE_STARTUP;
     bbr->bbr_round_count = 0;
     minmax_init(&bbr->bbr_max_bandwidth, 10);
@@ -200,13 +197,36 @@ lsquic_bbr_init (void *cong_ctl, const struct lsquic_conn_public *conn_pub,
     bbr->bbr_flags &= ~BBR_FLAG_LAST_SAMPLE_APP_LIMITED;
     bbr->bbr_flags &= ~BBR_FLAG_HAS_NON_APP_LIMITED;
     bbr->bbr_flags &= ~BBR_FLAG_FLEXIBLE_APP_LIMITED;
-
     set_startup_values(bbr);
+}
+
+
+static void
+lsquic_bbr_init (void *cong_ctl, const struct lsquic_conn_public *conn_pub,
+                                                enum quic_ft_bit retx_frames)
+{
+    struct lsquic_bbr *const bbr = cong_ctl;
+    bbr->bbr_conn_pub = conn_pub;
+    lsquic_bw_sampler_init(&bbr->bbr_bw_sampler, conn_pub->lconn, retx_frames);
+    bbr->bbr_rtt_stats = &conn_pub->rtt_stats;
+
+    init_bbr(bbr);
 
     LSQ_DEBUG("initialized");
 }
 
 
+static void
+lsquic_bbr_reinit (void *cong_ctl)
+{
+    struct lsquic_bbr *const bbr = cong_ctl;
+
+    init_bbr(bbr);
+
+    LSQ_DEBUG("re-initialized");
+}
+
+
 static lsquic_time_t
 get_min_rtt (const struct lsquic_bbr *bbr)
 {
@@ -665,13 +685,16 @@ on_exit_startup (struct lsquic_bbr *bbr, lsquic_time_t now)
 static void
 enter_probe_bw_mode (struct lsquic_bbr *bbr, lsquic_time_t now)
 {
+    uint8_t rand;
+
     set_mode(bbr, BBR_MODE_PROBE_BW);
     bbr->bbr_cwnd_gain = kCwndGain;
 
     // Pick a random offset for the gain cycle out of {0, 2..7} range. 1 is
     // excluded because in that case increased gain and decreased gain would not
     // follow each other.
-    bbr->bbr_cycle_current_offset = rand() % (kGainCycleLength - 1);
+    rand = lsquic_crand_get_byte(bbr->bbr_conn_pub->enpub->enp_crand);
+    bbr->bbr_cycle_current_offset = rand % (kGainCycleLength - 1);
     if (bbr->bbr_cycle_current_offset >= 1)
         ++bbr->bbr_cycle_current_offset;
 
@@ -1054,6 +1077,7 @@ const struct cong_ctl_if lsquic_cong_bbr_if =
     .cci_pacing_rate   = lsquic_bbr_pacing_rate,
     .cci_loss          = lsquic_bbr_loss,
     .cci_lost          = lsquic_bbr_lost,
+    .cci_reinit        = lsquic_bbr_reinit,
     .cci_timeout       = lsquic_bbr_timeout,
     .cci_sent          = lsquic_bbr_sent,
     .cci_was_quiet     = lsquic_bbr_was_quiet,
