@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_conn.h -- Connection interface
  *
@@ -35,7 +35,7 @@ enum lsquic_conn_flags {
     LSCONN_HAS_OUTGOING   = (1 << 1),
     LSCONN_HASHED         = (1 << 2),
     LSCONN_MINI           = (1 << 3),   /* This is a mini connection */
-    LSCONN_UNUSED_4       = (1 << 4),
+    LSCONN_IMMED_CLOSE    = (1 << 4),
     LSCONN_UNUSED_5       = (1 << 5),
     LSCONN_HANDSHAKE_DONE = (1 << 6),
     LSCONN_CLOSING        = (1 << 7),
@@ -52,9 +52,7 @@ enum lsquic_conn_flags {
     LSCONN_UNUSED_18      = (1 <<18),
     LSCONN_ATTQ           = (1 <<19),
     LSCONN_SKIP_ON_PROC   = (1 <<20),
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-    LSCONN_NO_CRYPTO      = (1 <<21),
-#endif
+    LSCONN_UNUSED_21      = (1 <<21),
     LSCONN_SERVER         = (1 <<22),
     LSCONN_IETF           = (1 <<23),
     LSCONN_RETRY_CONN     = (1 <<24),   /* This is a retry connection */
@@ -123,7 +121,7 @@ struct conn_iface
     (*ci_is_tickable) (struct lsquic_conn *);
 
     lsquic_time_t
-    (*ci_next_tick_time) (struct lsquic_conn *);
+    (*ci_next_tick_time) (struct lsquic_conn *, unsigned *why);
 
     int
     (*ci_can_write_ack) (struct lsquic_conn *);
@@ -158,9 +156,6 @@ struct conn_iface
     int
     (*ci_is_push_enabled) (struct lsquic_conn *);
 
-    struct lsquic_stream *
-    (*ci_get_stream_by_id) (struct lsquic_conn *, lsquic_stream_id_t stream_id);
-
     struct lsquic_engine *
     (*ci_get_engine) (struct lsquic_conn *);
 
@@ -288,9 +283,7 @@ struct lsquic_conn
     STAILQ_ENTRY(lsquic_conn)    cn_next_new_full;
     TAILQ_ENTRY(lsquic_conn)     cn_next_ticked;
     TAILQ_ENTRY(lsquic_conn)     cn_next_out;
-    TAILQ_ENTRY(lsquic_conn)     cn_next_susp_cert;
-    /* Reuse this entry, as evanecsent connections are never suspended: */
-#define cn_next_pr cn_next_susp_cert
+    TAILQ_ENTRY(lsquic_conn)     cn_next_pr;
     const struct conn_iface     *cn_if;
     const struct parse_funcs    *cn_pf;
     struct attq_elem            *cn_attq_elem;
@@ -364,7 +357,7 @@ struct conn_stats {
                             err_packets;        /* Error packets(?) */
         unsigned long       n_acks,
                             n_acks_proc,
-                            n_acks_merged[2];
+                            n_acks_merged;
         unsigned long       bytes;              /* Overall bytes in */
         unsigned long       headers_uncomp;     /* Sum of uncompressed header bytes */
         unsigned long       headers_comp;       /* Sum of compressed header bytes */
