@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_conn_public.h -- Connection's "public interface"
  *
@@ -54,6 +54,15 @@ struct lsquic_conn_public {
     struct conn_stats              *conn_stats;
 #endif
     const struct network_path      *path;
+#if LSQUIC_EXTRA_CHECKS
+    unsigned long                   stream_frame_bytes;
+#endif
+    /* "unsigned" is wide enough: these values are only used for amplification
+     * limit before initial path is validated.
+     */
+    unsigned                        bytes_in;   /* successfully processed */
+    unsigned                        bytes_out;
+    unsigned                        max_peer_ack_usec;
 };
 
 #endif
