@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_engine.c - QUIC engine
  */
@@ -73,12 +73,21 @@
 #include "lsquic_attq.h"
 #include "lsquic_min_heap.h"
 #include "lsquic_http1x_if.h"
-#include "lsquic_parse_common.h"
 #include "lsquic_handshake.h"
+#include "lsquic_crand.h"
+#include "lsquic_ietf.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_ENGINE
 #include "lsquic_logger.h"
 
+#ifndef LSQUIC_DEBUG_NEXT_ADV_TICK
+#define LSQUIC_DEBUG_NEXT_ADV_TICK 1
+#endif
+
+#if LSQUIC_DEBUG_NEXT_ADV_TICK
+#include "lsquic_alarmset.h"
+#endif
+
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 
 /* The batch of outgoing packets grows and shrinks dynamically */
@@ -131,9 +140,14 @@ force_close_conn (lsquic_engine_t *engine, lsquic_conn_t *conn);
 
 /* A connection can be referenced from one of six places:
  *
- *   1. A hash.  The engine maintains two hash tables -- one for full, and
- *      one for mini connections.  A connection starts its life in one of
- *      those.
+ *   1. A hash is used to find connections in order to dispatch an incoming
+ *      packet.  Connections can be hashed by CIDs or by address.  In the
+ *      former case, each connection has one or more mappings in the hash
+ *      table.  IETF QUIC connections have up to eight (in our implementation)
+ *      source CIDs and each of those would have a mapping.  In client mode,
+ *      depending on QUIC versions and options selected, it is may be
+ *      necessary to hash connections by address, in which case incoming
+ *      packets are delivered to connections based on the address.
  *
  *   2. Outgoing queue.
  *
@@ -207,13 +221,6 @@ struct lsquic_engine
     struct lsquic_hash                *conns_hash;
     struct min_heap                    conns_tickable;
     struct min_heap                    conns_out;
-    /* Use a union because only one iterator is being used at any one time */
-    union {
-        struct {
-            struct cert_susp_head *head;
-        }           resumed;
-        struct lsquic_conn *one_conn;
-    }                                  iter_state;
     struct eng_hist                    history;
     unsigned                           batch_size;
     struct pr_queue                   *pr_queue;
@@ -243,6 +250,13 @@ struct lsquic_engine
 #if LSQUIC_COUNT_ENGINE_CALLS
     unsigned long                      n_engine_calls;
 #endif
+#if LSQUIC_DEBUG_NEXT_ADV_TICK
+    uintptr_t                          last_logged_conn;
+    unsigned                           last_logged_ae_why;
+    int                                last_tick_diff;
+#endif
+    struct crand                       crand;
+    EVP_AEAD_CTX                       retry_aead_ctx;
 };
 
 
@@ -256,7 +270,6 @@ lsquic_engine_init_settings (struct lsquic_engine_settings *settings,
     {
         settings->es_cfcw        = LSQUIC_DF_CFCW_SERVER;
         settings->es_sfcw        = LSQUIC_DF_SFCW_SERVER;
-        settings->es_support_srej= LSQUIC_DF_SUPPORT_SREJ_SERVER;
         settings->es_init_max_data
                                  = LSQUIC_DF_INIT_MAX_DATA_SERVER;
         settings->es_init_max_stream_data_bidi_remote
@@ -273,7 +286,6 @@ lsquic_engine_init_settings (struct lsquic_engine_settings *settings,
     {
         settings->es_cfcw        = LSQUIC_DF_CFCW_CLIENT;
         settings->es_sfcw        = LSQUIC_DF_SFCW_CLIENT;
-        settings->es_support_srej= LSQUIC_DF_SUPPORT_SREJ_CLIENT;
         settings->es_init_max_data
                                  = LSQUIC_DF_INIT_MAX_DATA_CLIENT;
         settings->es_init_max_stream_data_bidi_remote
@@ -320,6 +332,10 @@ lsquic_engine_init_settings (struct lsquic_engine_settings *settings,
     settings->es_qpack_enc_max_size = LSQUIC_DF_QPACK_ENC_MAX_SIZE;
     settings->es_qpack_enc_max_blocked = LSQUIC_DF_QPACK_ENC_MAX_BLOCKED;
     settings->es_allow_migration = LSQUIC_DF_ALLOW_MIGRATION;
+    settings->es_ql_bits         = LSQUIC_DF_QL_BITS;
+    settings->es_spin            = LSQUIC_DF_SPIN;
+    settings->es_delayed_acks    = LSQUIC_DF_DELAYED_ACKS;
+    settings->es_timestamps      = LSQUIC_DF_TIMESTAMPS;
 }
 
 
@@ -370,13 +386,12 @@ lsquic_engine_check_settings (const struct lsquic_engine_settings *settings,
                         "The maximum value of idle timeout is 600 seconds");
         return -1;
     }
-    if (!(!(flags & ENG_SERVER) && settings->es_scid_len == 0)
-            && (settings->es_scid_len < 4 || settings->es_scid_len > 18))
+    if (settings->es_scid_len > MAX_CID_LEN)
     {
         if (err_buf)
             snprintf(err_buf, err_buf_sz, "Source connection ID cannot be %u "
-                        "bytes long; it must be between 4 and 18.",
-                        settings->es_scid_len);
+                        "bytes long; it must be between 0 and %u.",
+                        settings->es_scid_len, MAX_CID_LEN);
         return -1;
     }
 
@@ -387,6 +402,23 @@ lsquic_engine_check_settings (const struct lsquic_engine_settings *settings,
                 "algorithm value %u", settings->es_cc_algo);
         return -1;
     }
+
+    if (!(settings->es_ql_bits >= 0 && settings->es_ql_bits <= 2))
+    {
+        if (err_buf)
+            snprintf(err_buf, err_buf_sz, "Invalid QL bits value %d ",
+                settings->es_ql_bits);
+        return -1;
+    }
+
+    if (!(settings->es_spin == 0 || settings->es_spin == 1))
+    {
+        if (err_buf)
+            snprintf(err_buf, err_buf_sz, "Invalid spin value %d",
+                settings->es_spin);
+        return -1;
+    }
+
     return 0;
 }
 
@@ -432,6 +464,7 @@ lsquic_engine_new (unsigned flags,
                    const struct lsquic_engine_api *api)
 {
     lsquic_engine_t *engine;
+    size_t alpn_len;
     char err_buf[100];
 
     if (!api->ea_packets_out)
@@ -440,6 +473,24 @@ lsquic_engine_new (unsigned flags,
         return NULL;
     }
 
+    if (!api->ea_stream_if)
+    {
+        LSQ_ERROR("stream interface is not specified");
+        return NULL;
+    }
+
+    if (!(flags & LSENG_HTTP) && api->ea_alpn)
+    {
+        alpn_len = strlen(api->ea_alpn);
+        if (alpn_len < 1 || alpn_len > 255)
+        {
+            LSQ_ERROR("ALPN string length invalid: %zd bytes", alpn_len);
+            return NULL;
+        }
+    }
+    else
+        alpn_len = 0;
+
     if (api->ea_settings &&
                 0 != lsquic_engine_check_settings(api->ea_settings, flags,
                                                     err_buf, sizeof(err_buf)))
@@ -532,11 +583,12 @@ lsquic_engine_new (unsigned flags,
     if (hash_conns_by_addr(engine))
         engine->flags |= ENG_CONNS_BY_ADDR;
     engine->conns_hash = lsquic_hash_create();
+    engine->pub.enp_tokgen = lsquic_tg_new(&engine->pub);
+    if (!engine->pub.enp_tokgen)
+        return NULL;
+    engine->pub.enp_crand = &engine->crand;
     if (flags & ENG_SERVER)
     {
-        engine->pub.enp_tokgen = lsquic_tg_new(&engine->pub);
-        if (!engine->pub.enp_tokgen)
-            return NULL;
         engine->pr_queue = prq_create(
             10000 /* TODO: make configurable */, MAX_OUT_BATCH_SIZE,
             &engine->pub);
@@ -567,6 +619,21 @@ lsquic_engine_new (unsigned flags,
         }
     }
 
+    if (alpn_len)
+    {
+        engine->pub.enp_alpn = malloc(alpn_len + 1);
+        if (!engine->pub.enp_alpn)
+        {
+            lsquic_engine_destroy(engine);
+            return NULL;
+        }
+        engine->pub.enp_alpn[0] = alpn_len;
+        memcpy(engine->pub.enp_alpn + 1, api->ea_alpn, alpn_len);
+    }
+
+    if (flags & LSENG_HTTP)
+        engine->pub.enp_flags |= ENPUB_HTTP;
+
 #ifndef NDEBUG
     {
         const char *env;
@@ -601,6 +668,14 @@ lsquic_engine_new (unsigned flags,
 #if LSQUIC_CONN_STATS
     engine->stats_fh = api->ea_stats_fh;
 #endif
+    if (1 != EVP_AEAD_CTX_init(&engine->retry_aead_ctx, EVP_aead_aes_128_gcm(),
+                            IETF_RETRY_KEY_BUF, IETF_RETRY_KEY_SZ, 16, NULL))
+    {
+        LSQ_ERROR("could not initialize retry AEAD ctx");
+        lsquic_engine_destroy(engine);
+        return NULL;
+    }
+    engine->pub.enp_retry_aead_ctx = &engine->retry_aead_ctx;
 
     LSQ_INFO("instantiated engine");
     return engine;
@@ -711,7 +786,8 @@ destroy_conn (struct lsquic_engine *engine, struct lsquic_conn *conn,
             && (conn->cn_flags & (LSCONN_MINI|LSCONN_PROMOTED))
                                         != (LSCONN_MINI|LSCONN_PROMOTED))
     {
-        if (conn->cn_if->ci_drain_time &&
+        if (!(conn->cn_flags & LSCONN_IMMED_CLOSE)
+            && conn->cn_if->ci_drain_time &&
             (drain_time = conn->cn_if->ci_drain_time(conn), drain_time))
         {
             for (cce = cce_iter_first(&citer, conn); cce;
@@ -1007,10 +1083,9 @@ find_conn (lsquic_engine_t *engine, lsquic_packet_in_t *packet_in,
 static lsquic_conn_t *
 find_or_create_conn (lsquic_engine_t *engine, lsquic_packet_in_t *packet_in,
          struct packin_parse_state *ppstate, const struct sockaddr *sa_local,
-         const struct sockaddr *sa_peer, void *peer_ctx)
+         const struct sockaddr *sa_peer, void *peer_ctx, size_t packet_in_size)
 {
     struct lsquic_hash_elem *el;
-    lsquic_cid_t odcid;
     struct purga_el *puel;
     lsquic_conn_t *conn;
 
@@ -1120,7 +1195,7 @@ find_or_create_conn (lsquic_engine_t *engine, lsquic_packet_in_t *packet_in,
     if ((1 << version) & LSQUIC_IETF_VERSIONS)
     {
         conn = lsquic_mini_conn_ietf_new(&engine->pub, packet_in, version,
-                    sa_peer->sa_family == AF_INET, odcid.len ? &odcid : NULL);
+                    sa_peer->sa_family == AF_INET, NULL, packet_in_size);
     }
     else
     {
@@ -1179,7 +1254,7 @@ lsquic_engine_add_conn_to_tickable (struct lsquic_engine_public *enpub,
 
 void
 lsquic_engine_add_conn_to_attq (struct lsquic_engine_public *enpub,
-                                lsquic_conn_t *conn, lsquic_time_t tick_time)
+                    lsquic_conn_t *conn, lsquic_time_t tick_time, unsigned why)
 {
     lsquic_engine_t *const engine = (lsquic_engine_t *) enpub;
     if (conn->cn_flags & LSCONN_TICKABLE)
@@ -1194,11 +1269,11 @@ lsquic_engine_add_conn_to_attq (struct lsquic_engine_public *enpub,
         if (lsquic_conn_adv_time(conn) != tick_time)
         {
             attq_remove(engine->attq, conn);
-            if (0 != attq_add(engine->attq, conn, tick_time))
+            if (0 != attq_add(engine->attq, conn, tick_time, why))
                 engine_decref_conn(engine, conn, LSCONN_ATTQ);
         }
     }
-    else if (0 == attq_add(engine->attq, conn, tick_time))
+    else if (0 == attq_add(engine->attq, conn, tick_time, why))
         engine_incref_conn(conn, LSCONN_ATTQ);
 }
 
@@ -1231,11 +1306,10 @@ find_conn_by_srst (struct lsquic_engine *engine,
 static int
 process_packet_in (lsquic_engine_t *engine, lsquic_packet_in_t *packet_in,
        struct packin_parse_state *ppstate, const struct sockaddr *sa_local,
-       const struct sockaddr *sa_peer, void *peer_ctx)
+       const struct sockaddr *sa_peer, void *peer_ctx, size_t packet_in_size)
 {
     lsquic_conn_t *conn;
     const unsigned char *packet_in_data;
-    size_t packet_in_size;
 
     if (lsquic_packet_in_is_gquic_prst(packet_in)
                                 && !engine->pub.enp_settings.es_honor_prst)
@@ -1247,13 +1321,14 @@ process_packet_in (lsquic_engine_t *engine, lsquic_packet_in_t *packet_in,
 
     if (engine->flags & ENG_SERVER)
         conn = find_or_create_conn(engine, packet_in, ppstate, sa_local,
-                                                        sa_peer, peer_ctx);
+                                            sa_peer, peer_ctx, packet_in_size);
     else
         conn = find_conn(engine, packet_in, ppstate, sa_local);
 
     if (!conn)
     {
         if (engine->pub.enp_settings.es_honor_prst
+                && packet_in_size == packet_in->pi_data_sz /* Full UDP packet */
                 && !(packet_in->pi_flags & PI_GQUIC)
                 && engine->pub.enp_srst_hash
                 && (conn = find_conn_by_srst(engine, packet_in)))
@@ -1345,6 +1420,9 @@ lsquic_engine_destroy (lsquic_engine_t *engine)
     }
     lsquic_hash_destroy(engine->conns_hash);
 
+    while ((conn = attq_pop(engine->attq, UINT64_MAX)))
+        (void) engine_decref_conn(engine, conn, LSCONN_ATTQ);
+
     assert(0 == engine->n_conns);
     assert(0 == engine->mini_conns_count);
     if (engine->pr_queue)
@@ -1410,6 +1488,9 @@ lsquic_engine_destroy (lsquic_engine_t *engine)
 #if LSQUIC_COUNT_ENGINE_CALLS
     LSQ_NOTICE("number of calls into the engine: %lu", engine->n_engine_calls);
 #endif
+    if (engine->pub.enp_retry_aead_ctx)
+        EVP_AEAD_CTX_cleanup(engine->pub.enp_retry_aead_ctx);
+    free(engine->pub.enp_alpn);
     free(engine);
 }
 
@@ -1459,7 +1540,8 @@ add_conn_to_hash (struct lsquic_engine *engine, struct lsquic_conn *conn,
 
 
 lsquic_conn_t *
-lsquic_engine_connect (lsquic_engine_t *engine, const struct sockaddr *local_sa,
+lsquic_engine_connect (lsquic_engine_t *engine, enum lsquic_version version,
+                       const struct sockaddr *local_sa,
                        const struct sockaddr *peer_sa,
                        void *peer_ctx, lsquic_conn_ctx_t *conn_ctx, 
                        const char *hostname, unsigned short max_packet_size,
@@ -1467,7 +1549,7 @@ lsquic_engine_connect (lsquic_engine_t *engine, const struct sockaddr *local_sa,
                        const unsigned char *token, size_t token_sz)
 {
     lsquic_conn_t *conn;
-    unsigned flags;
+    unsigned flags, versions;
     int is_ipv4;
 
     ENGINE_IN(engine);
@@ -1489,13 +1571,31 @@ lsquic_engine_connect (lsquic_engine_t *engine, const struct sockaddr *local_sa,
         return NULL;
     flags = engine->flags & (ENG_SERVER|ENG_HTTP);
     is_ipv4 = peer_sa->sa_family == AF_INET;
-    if (engine->pub.enp_settings.es_versions & LSQUIC_IETF_VERSIONS)
-        conn = lsquic_ietf_full_conn_client_new(&engine->pub,
+    if (zero_rtt && zero_rtt_len)
+    {
+        version = lsquic_zero_rtt_version(zero_rtt, zero_rtt_len);
+        if (version >= N_LSQVER)
+        {
+            LSQ_INFO("zero-rtt version is bad, won't use");
+            zero_rtt = NULL;
+            zero_rtt_len = 0;
+        }
+    }
+    if (version >= N_LSQVER)
+    {
+        if (version > N_LSQVER)
+            LSQ_WARN("invalid version specified, engine will pick");
+        versions = engine->pub.enp_settings.es_versions;
+    }
+    else
+        versions = 1u << version;
+    if (versions & LSQUIC_IETF_VERSIONS)
+        conn = lsquic_ietf_full_conn_client_new(&engine->pub, versions,
                     flags, hostname, max_packet_size,
                     is_ipv4, zero_rtt, zero_rtt_len, token, token_sz);
     else
-        conn = lsquic_gquic_full_conn_client_new(&engine->pub, flags,
-                            hostname, max_packet_size, is_ipv4,
+        conn = lsquic_gquic_full_conn_client_new(&engine->pub, versions,
+                            flags, hostname, max_packet_size, is_ipv4,
                             zero_rtt, zero_rtt_len);
     if (!conn)
         goto err;
@@ -1540,7 +1640,7 @@ remove_conn_from_hash (lsquic_engine_t *engine, lsquic_conn_t *conn)
 
 
 static void
-refflags2str (enum lsquic_conn_flags flags, char s[6])
+refflags2str (enum lsquic_conn_flags flags, char s[7])
 {
     *s = 'C'; s += !!(flags & LSCONN_CLOSING);
     *s = 'H'; s += !!(flags & LSCONN_HASHED);
@@ -1555,13 +1655,12 @@ refflags2str (enum lsquic_conn_flags flags, char s[6])
 static void
 engine_incref_conn (lsquic_conn_t *conn, enum lsquic_conn_flags flag)
 {
-    const lsquic_cid_t *cid;
     char str[2][7];
     assert(flag & CONN_REF_FLAGS);
     assert(!(conn->cn_flags & flag));
     conn->cn_flags |= flag;
-    cid = lsquic_conn_log_cid(conn);
-    LSQ_DEBUGC("incref conn %"CID_FMT", '%s' -> '%s'", CID_BITS(cid),
+    LSQ_DEBUGC("incref conn %"CID_FMT", '%s' -> '%s'",
+                    CID_BITS(lsquic_conn_log_cid(conn)),
                     (refflags2str(conn->cn_flags & ~flag, str[0]), str[0]),
                     (refflags2str(conn->cn_flags, str[1]), str[1]));
 }
@@ -1571,7 +1670,6 @@ static lsquic_conn_t *
 engine_decref_conn (lsquic_engine_t *engine, lsquic_conn_t *conn,
                                         enum lsquic_conn_flags flags)
 {
-    const lsquic_cid_t *cid;
     char str[2][7];
     lsquic_time_t now;
     assert(flags & CONN_REF_FLAGS);
@@ -1581,8 +1679,8 @@ engine_decref_conn (lsquic_engine_t *engine, lsquic_conn_t *conn,
         assert(0 == (conn->cn_flags & LSCONN_HASHED));
 #endif
     conn->cn_flags &= ~flags;
-    cid = lsquic_conn_log_cid(conn);
-    LSQ_DEBUGC("decref conn %"CID_FMT", '%s' -> '%s'", CID_BITS(cid),
+    LSQ_DEBUGC("decref conn %"CID_FMT", '%s' -> '%s'",
+                    CID_BITS(lsquic_conn_log_cid(conn)),
                     (refflags2str(conn->cn_flags | flags, str[0]), str[0]),
                     (refflags2str(conn->cn_flags, str[1]), str[1]));
     if (0 == (conn->cn_flags & CONN_REF_FLAGS))
@@ -1922,7 +2020,11 @@ coi_reheap (struct conns_out_iter *iter, lsquic_engine_t *engine)
     {
         TAILQ_REMOVE(&iter->coi_active_list, conn, cn_next_out);
         conn->cn_flags &= ~LSCONN_COI_ACTIVE;
-        lsquic_mh_insert(iter->coi_heap, conn, conn->cn_last_sent);
+        if ((conn->cn_flags & CONN_REF_FLAGS) != LSCONN_HAS_OUTGOING
+                                && !(conn->cn_flags & LSCONN_IMMED_CLOSE))
+            lsquic_mh_insert(iter->coi_heap, conn, conn->cn_last_sent);
+        else    /* Closed connection gets one shot at sending packets */
+            (void) engine_decref_conn(engine, conn, LSCONN_HAS_OUTGOING);
     }
     while ((conn = TAILQ_FIRST(&iter->coi_inactive_list)))
     {
@@ -1969,14 +2071,67 @@ lose_matching_packets (const lsquic_engine_t *engine, struct out_batch *batch,
 #define CONST_BATCH
 #endif
 
-static unsigned
-send_batch (lsquic_engine_t *engine, struct conns_out_iter *conns_iter,
-            CONST_BATCH struct out_batch *batch, unsigned n_to_send)
+
+struct send_batch_ctx {
+    struct conns_stailq                 *closed_conns;
+    struct conns_tailq                  *ticked_conns;
+    struct conns_out_iter               *conns_iter;
+    CONST_BATCH struct out_batch        *batch;
+};
+
+
+static void
+close_conn_immediately (struct lsquic_engine *engine,
+                const struct send_batch_ctx *sb_ctx, struct lsquic_conn *conn)
 {
-    int n_sent, i;
+    conn->cn_flags |= LSCONN_IMMED_CLOSE;
+    if (!(conn->cn_flags & LSCONN_CLOSING))
+    {
+        STAILQ_INSERT_TAIL(sb_ctx->closed_conns, conn, cn_next_closed_conn);
+        engine_incref_conn(conn, LSCONN_CLOSING);
+        if (conn->cn_flags & LSCONN_HASHED)
+            remove_conn_from_hash(engine, conn);
+    }
+    if (conn->cn_flags & LSCONN_TICKED)
+    {
+        TAILQ_REMOVE(sb_ctx->ticked_conns, conn, cn_next_ticked);
+        engine_decref_conn(engine, conn, LSCONN_TICKED);
+    }
+}
+
+
+static void
+close_conn_on_send_error (struct lsquic_engine *engine,
+                          const struct send_batch_ctx *sb_ctx, int n, int e_val)
+{
+    const struct out_batch *batch = sb_ctx->batch;
+    struct lsquic_conn *const conn = batch->conns[n];
+    char buf[2][INET6_ADDRSTRLEN + sizeof(":65535")];
+
+    LSQ_WARNC("error sending packet for %s connection %"CID_FMT" - close it; "
+        "src: %s; dst: %s; errno: %d",
+        conn->cn_flags & LSCONN_EVANESCENT ? "evanecsent" :
+        conn->cn_flags & LSCONN_MINI ? "mini" : "regular",
+        CID_BITS(lsquic_conn_log_cid(conn)),
+        SA2STR(batch->outs[n].local_sa, buf[0]),
+        SA2STR(batch->outs[n].dest_sa, buf[1]),
+        e_val);
+    if (conn->cn_flags & LSCONN_EVANESCENT)
+        lsquic_prq_drop(conn);
+    else
+        close_conn_immediately(engine, sb_ctx, conn);
+}
+
+
+static unsigned
+send_batch (lsquic_engine_t *engine, const struct send_batch_ctx *sb_ctx,
+            unsigned n_to_send)
+{
+    int n_sent, i, e_val;
     lsquic_time_t now;
     unsigned off;
     size_t count;
+    CONST_BATCH struct out_batch *const batch = sb_ctx->batch;
     struct lsquic_packet_out *CONST_BATCH *packet_out, *CONST_BATCH *end;
 
 #ifndef NDEBUG
@@ -1998,18 +2153,22 @@ send_batch (lsquic_engine_t *engine, struct conns_out_iter *conns_iter,
     }
     n_sent = engine->packets_out(engine->packets_out_ctx, batch->outs,
                                                                 n_to_send);
+    e_val = errno;
     if (n_sent < (int) n_to_send)
     {
         engine->pub.enp_flags &= ~ENPUB_CAN_SEND;
         engine->resume_sending_at = now + 1000000;
         LSQ_DEBUG("cannot send packets");
         EV_LOG_GENERIC_EVENT("cannot send packets");
+        if (!(EAGAIN == e_val || EWOULDBLOCK == e_val))
+            close_conn_on_send_error(engine, sb_ctx,
+                                        n_sent < 0 ? 0 : n_sent, e_val);
     }
     if (n_sent >= 0)
         LSQ_DEBUG("packets out returned %d (out of %u)", n_sent, n_to_send);
     else
     {
-        LSQ_DEBUG("packets out returned an error: %s", strerror(errno));
+        LSQ_DEBUG("packets out returned an error: %s", strerror(e_val));
         n_sent = 0;
     }
     if (n_sent > 0)
@@ -2074,7 +2233,7 @@ send_batch (lsquic_engine_t *engine, struct conns_out_iter *conns_iter,
                                                                 *packet_out);
         while (--packet_out > end);
         if (!(batch->conns[i]->cn_flags & (LSCONN_COI_ACTIVE|LSCONN_EVANESCENT)))
-            coi_reactivate(conns_iter, batch->conns[i]);
+            coi_reactivate(sb_ctx->conns_iter, batch->conns[i]);
     }
     return n_sent;
 }
@@ -2113,16 +2272,11 @@ iov_size (const struct iovec *iov, const struct iovec *const end)
 }
 
 
-/* XXX A lot of extra setup -- two extra arguments to this function, two extra
- * connection ref flags and queues -- is just to handle the ENCPA_BADCRYPT case,
- * which never really happens.
- */
 static void
 send_packets_out (struct lsquic_engine *engine,
                   struct conns_tailq *ticked_conns,
                   struct conns_stailq *closed_conns)
 {
-    const lsquic_cid_t *cid;
     unsigned n, w, n_sent, n_batches_sent;
     lsquic_packet_out_t *packet_out;
     struct lsquic_packet_out **packet;
@@ -2131,6 +2285,12 @@ send_packets_out (struct lsquic_engine *engine,
     struct iovec *iov, *packet_iov;
     struct conns_out_iter conns_iter;
     int shrink, deadline_exceeded;
+    const struct send_batch_ctx sb_ctx = {
+        closed_conns,
+        ticked_conns,
+        &conns_iter,
+        &engine->out_batch,
+    };
 
     coi_init(&conns_iter, engine);
     n_batches_sent = 0;
@@ -2142,14 +2302,13 @@ send_packets_out (struct lsquic_engine *engine,
 
     while ((conn = coi_next(&conns_iter)))
     {
-        cid = lsquic_conn_log_cid(conn);
         packet_out = conn->cn_if->ci_next_packet_to_send(conn, 0);
         if (!packet_out) {
             /* Evanescent connection always has a packet to send: */
             assert(!(conn->cn_flags & LSCONN_EVANESCENT));
             LSQ_DEBUGC("batched all outgoing packets for %s conn %"CID_FMT,
-                (conn->cn_flags & LSCONN_MINI   ? "mini"   :
-                                          "full"), CID_BITS(cid));
+                (conn->cn_flags & LSCONN_MINI   ? "mini" : "full"),
+                CID_BITS(lsquic_conn_log_cid(conn)));
             coi_deactivate(&conns_iter, conn);
             continue;
         }
@@ -2168,22 +2327,11 @@ send_packets_out (struct lsquic_engine *engine,
                 /* This is pretty bad: close connection immediately */
                 conn->cn_if->ci_packet_not_sent(conn, packet_out);
                 LSQ_INFOC("conn %"CID_FMT" has unsendable packets",
-                                                    CID_BITS(cid));
+                                        CID_BITS(lsquic_conn_log_cid(conn)));
                 if (!(conn->cn_flags & LSCONN_EVANESCENT))
                 {
-                    if (!(conn->cn_flags & LSCONN_CLOSING))
-                    {
-                        STAILQ_INSERT_TAIL(closed_conns, conn, cn_next_closed_conn);
-                        engine_incref_conn(conn, LSCONN_CLOSING);
-                        if (conn->cn_flags & LSCONN_HASHED)
-                            remove_conn_from_hash(engine, conn);
-                    }
+                    close_conn_immediately(engine, &sb_ctx, conn);
                     coi_deactivate(&conns_iter, conn);
-                    if (conn->cn_flags & LSCONN_TICKED)
-                    {
-                        TAILQ_REMOVE(ticked_conns, conn, cn_next_ticked);
-                        engine_decref_conn(engine, conn, LSCONN_TICKED);
-                    }
                 }
                 continue;
             case ENCPA_OK:
@@ -2201,7 +2349,7 @@ send_packets_out (struct lsquic_engine *engine,
             }
         }
         LSQ_DEBUGC("batched packet %"PRIu64" for connection %"CID_FMT,
-                                        packet_out->po_packno, CID_BITS(cid));
+                    packet_out->po_packno, CID_BITS(lsquic_conn_log_cid(conn)));
         if (packet_out->po_flags & PO_ENCRYPTED)
         {
             iov->iov_base          = packet_out->po_enc_data;
@@ -2242,7 +2390,7 @@ send_packets_out (struct lsquic_engine *engine,
         if (n == engine->batch_size
             || iov >= batch->iov + sizeof(batch->iov) / sizeof(batch->iov[0]))
         {
-            w = send_batch(engine, &conns_iter, batch, n);
+            w = send_batch(engine, &sb_ctx, n);
             n = 0;
             iov = batch->iov;
             packet = batch->packets;
@@ -2262,7 +2410,7 @@ send_packets_out (struct lsquic_engine *engine,
   end_for:
 
     if (n > 0) {
-        w = send_batch(engine, &conns_iter, batch, n);
+        w = send_batch(engine, &sb_ctx, n);
         n_sent += w;
         shrink = w < n;
         ++n_batches_sent;
@@ -2300,7 +2448,32 @@ reset_deadline (lsquic_engine_t *engine, lsquic_time_t now)
 }
 
 
-/* TODO: this is a user-facing function, account for load */
+static void
+check_tickable_conns_again (struct lsquic_engine *engine)
+{
+    struct lsquic_hash_elem *el;
+    struct lsquic_conn *conn;
+    unsigned count;
+
+    count = 0;
+    for (el = lsquic_hash_first(engine->conns_hash); el;
+                                el = lsquic_hash_next(engine->conns_hash))
+    {
+        conn = lsquic_hashelem_getdata(el);
+        if (!(conn->cn_flags & LSCONN_TICKABLE)
+            && conn->cn_if->ci_is_tickable(conn))
+        {
+            lsquic_mh_insert(&engine->conns_tickable, conn,
+                                                    conn->cn_last_ticked);
+            engine_incref_conn(conn, LSCONN_TICKABLE);
+            ++count;
+        }
+    }
+    LSQ_DEBUG("%u connection%s tickable again after sending has been "
+        "re-enabled", count, count == 1 ? " is" : "s are");
+}
+
+
 void
 lsquic_engine_send_unsent_packets (lsquic_engine_t *engine)
 {
@@ -2318,6 +2491,7 @@ lsquic_engine_send_unsent_packets (lsquic_engine_t *engine)
         LSQ_DEBUG("can send again");
         EV_LOG_GENERIC_EVENT("can send again");
         engine->pub.enp_flags |= ENPUB_CAN_SEND;
+        check_tickable_conns_again(engine);
     }
 
     send_packets_out(engine, &ticked_conns, &closed_conns);
@@ -2352,7 +2526,7 @@ process_connections (lsquic_engine_t *engine, conn_iter_f next_conn,
 {
     lsquic_conn_t *conn;
     enum tick_st tick_st;
-    unsigned i;
+    unsigned i, why;
     lsquic_time_t next_tick_time;
     struct conns_stailq closed_conns;
     struct conns_tailq ticked_conns;
@@ -2395,9 +2569,9 @@ process_connections (lsquic_engine_t *engine, conn_iter_f next_conn,
                 STAILQ_INSERT_TAIL(&new_full_conns, new_conn, cn_next_new_full);
                 new_conn->cn_last_sent = engine->last_sent;
                 eng_hist_inc(&engine->history, now, sl_new_full_conns);
+                conn->cn_flags |= LSCONN_PROMOTED;
             }
             tick_st |= TICK_CLOSE;  /* Destroy mini connection */
-            conn->cn_flags |= LSCONN_PROMOTED;
         }
         if (tick_st & TICK_SEND)
         {
@@ -2435,9 +2609,6 @@ process_connections (lsquic_engine_t *engine, conn_iter_f next_conn,
         (void) engine_decref_conn(engine, conn, LSCONN_CLOSING);
     }
 
-    /* TODO Heapification can be optimized by switching to the Floyd method:
-     * https://en.wikipedia.org/wiki/Binary_heap#Building_a_heap
-     */
     while ((conn = TAILQ_FIRST(&ticked_conns)))
     {
         TAILQ_REMOVE(&ticked_conns, conn, cn_next_ticked);
@@ -2445,19 +2616,24 @@ process_connections (lsquic_engine_t *engine, conn_iter_f next_conn,
         if (!(conn->cn_flags & LSCONN_TICKABLE)
             && conn->cn_if->ci_is_tickable(conn))
         {
+            /* Floyd heapification is not faster, don't bother. */
             lsquic_mh_insert(&engine->conns_tickable, conn, conn->cn_last_ticked);
             engine_incref_conn(conn, LSCONN_TICKABLE);
         }
         else if (!(conn->cn_flags & LSCONN_ATTQ))
         {
-            next_tick_time = conn->cn_if->ci_next_tick_time(conn);
+            next_tick_time = conn->cn_if->ci_next_tick_time(conn, &why);
             if (next_tick_time)
             {
-                if (0 == attq_add(engine->attq, conn, next_tick_time))
+                if (0 == attq_add(engine->attq, conn, next_tick_time, why))
                     engine_incref_conn(conn, LSCONN_ATTQ);
             }
             else
-                assert(0);
+                /* In all other cases, the idle timeout would make the next
+                 * tick time non-zero:
+                 */
+                assert((conn->cn_flags & LSCONN_IETF)
+                    && engine->pub.enp_settings.es_idle_timeout == 0);
         }
     }
 
@@ -2488,8 +2664,7 @@ lsquic_engine_packet_in (lsquic_engine_t *engine,
 
     if (engine->flags & ENG_SERVER)
         parse_packet_in_begin = lsquic_parse_packet_in_server_begin;
-    else
-    if (engine->flags & ENG_CONNS_BY_ADDR)
+    else if (engine->flags & ENG_CONNS_BY_ADDR)
     {
         struct lsquic_hash_elem *el;
         const struct lsquic_conn *conn;
@@ -2501,6 +2676,8 @@ lsquic_engine_packet_in (lsquic_engine_t *engine,
             parse_packet_in_begin = lsquic_gquic_parse_packet_in_begin;
         else if ((1 << conn->cn_version) & LSQUIC_IETF_VERSIONS)
             parse_packet_in_begin = lsquic_ietf_v1_parse_packet_in_begin;
+        else if (conn->cn_version == LSQVER_050)
+            parse_packet_in_begin = lsquic_Q050_parse_packet_in_begin;
         else
         {
             assert(conn->cn_version == LSQVER_046
@@ -2532,8 +2709,8 @@ lsquic_engine_packet_in (lsquic_engine_t *engine,
         {
             LSQ_DEBUG("Cannot parse incoming packet's header");
             lsquic_mm_put_packet_in(&engine->pub.enp_mm, packet_in);
-            errno = EINVAL;
-            return -1;
+            s = 1;
+            break;
         }
 
         packet_in_data += packet_in->pi_data_sz;
@@ -2541,7 +2718,7 @@ lsquic_engine_packet_in (lsquic_engine_t *engine,
         packet_in->pi_flags |= (3 & ecn) << PIBIT_ECN_SHIFT;
         eng_hist_inc(&engine->history, packet_in->pi_received, sl_packets_in);
         s = process_packet_in(engine, packet_in, &ppstate, sa_local, sa_peer,
-                                                                    peer_ctx);
+                            peer_ctx, packet_in_size);
         n_zeroes += s == 0;
     }
     while (0 == s && packet_in_data < packet_end);
@@ -2587,38 +2764,105 @@ lsquic_engine_cooldown (lsquic_engine_t *engine)
 int
 lsquic_engine_earliest_adv_tick (lsquic_engine_t *engine, int *diff)
 {
-    const lsquic_time_t *next_attq_time;
+    const struct attq_elem *next_attq;
     lsquic_time_t now, next_time;
+#if LSQUIC_DEBUG_NEXT_ADV_TICK
+    const struct lsquic_conn *conn;
+    const enum lsq_log_level L = LSQ_LOG_DEBUG;  /* Easy toggle */
+#endif
 
     ENGINE_CALLS_INCR(engine);
 
-    if (((engine->flags & ENG_PAST_DEADLINE)
+    if ((engine->flags & ENG_PAST_DEADLINE)
                                     && lsquic_mh_count(&engine->conns_out))
-        || (engine->pr_queue && prq_have_pending(engine->pr_queue))
-        || lsquic_mh_count(&engine->conns_tickable))
     {
+#if LSQUIC_DEBUG_NEXT_ADV_TICK
+        conn = lsquic_mh_peek(&engine->conns_out);
+        engine->last_logged_conn = 0;
+        LSQ_LOGC(L, "next advisory tick is now: went past deadline last time "
+            "and have %u outgoing connection%.*s (%"CID_FMT" first)",
+            lsquic_mh_count(&engine->conns_out),
+            lsquic_mh_count(&engine->conns_out) != 1, "s",
+            CID_BITS(lsquic_conn_log_cid(conn)));
+#endif
         *diff = 0;
         return 1;
     }
 
-    next_attq_time = attq_next_time(engine->attq);
+    if (engine->pr_queue && prq_have_pending(engine->pr_queue))
+    {
+#if LSQUIC_DEBUG_NEXT_ADV_TICK
+        engine->last_logged_conn = 0;
+        LSQ_LOG(L, "next advisory tick is now: have pending PRQ elements");
+#endif
+        *diff = 0;
+        return 1;
+    }
+
+    if (lsquic_mh_count(&engine->conns_tickable))
+    {
+#if LSQUIC_DEBUG_NEXT_ADV_TICK
+        conn = lsquic_mh_peek(&engine->conns_tickable);
+        engine->last_logged_conn = 0;
+        LSQ_LOGC(L, "next advisory tick is now: have %u tickable "
+            "connection%.*s (%"CID_FMT" first)",
+            lsquic_mh_count(&engine->conns_tickable),
+            lsquic_mh_count(&engine->conns_tickable) != 1, "s",
+            CID_BITS(lsquic_conn_log_cid(conn)));
+#endif
+        *diff = 0;
+        return 1;
+    }
+
+    next_attq = attq_next(engine->attq);
     if (engine->pub.enp_flags & ENPUB_CAN_SEND)
     {
-        if (next_attq_time)
-            next_time = *next_attq_time;
+        if (next_attq)
+            next_time = next_attq->ae_adv_time;
         else
             return 0;
     }
     else
     {
-        if (next_attq_time)
-            next_time = MIN(*next_attq_time, engine->resume_sending_at);
+        if (next_attq)
+        {
+            next_time = next_attq->ae_adv_time;
+            if (engine->resume_sending_at < next_time)
+            {
+                next_time = engine->resume_sending_at;
+                next_attq = NULL;
+            }
+        }
         else
             next_time = engine->resume_sending_at;
     }
 
     now = lsquic_time_now();
     *diff = (int) ((int64_t) next_time - (int64_t) now);
+#if LSQUIC_DEBUG_NEXT_ADV_TICK
+    if (next_attq)
+    {
+        /* Deduplicate consecutive log messages about the same reason for the
+         * same connection.
+         * If diff is always zero or diff reset to a higher value, event is
+         * still logged.
+         */
+        if (!((unsigned) next_attq->ae_why == engine->last_logged_ae_why
+                    && (uintptr_t) next_attq->ae_conn
+                                            == engine->last_logged_conn
+                    && *diff < engine->last_tick_diff))
+        {
+            engine->last_logged_conn = (uintptr_t) next_attq->ae_conn;
+            engine->last_logged_ae_why = (unsigned) next_attq->ae_why;
+            engine->last_tick_diff = *diff;
+            LSQ_LOGC(L, "next advisory tick is %d usec away: conn %"CID_FMT
+                ": %s", *diff, CID_BITS(lsquic_conn_log_cid(next_attq->ae_conn)),
+                lsquic_attq_why2str(next_attq->ae_why));
+        }
+    }
+    else
+        LSQ_LOG(L, "next advisory tick is %d usec away: resume sending", *diff);
+#endif
     return 1;
 }
 
