@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef WIN32
 #include <arpa/inet.h>
 #else
@@ -66,13 +66,24 @@ lsquic_ev_log_packet_in (const lsquic_cid_t *cid,
             (unsigned) (packet_in->pi_data_sz + GQUIC_PACKET_HASH_SZ));
         break;
     default:
-        LCID("packet in: %"PRIu64", type: %s, size: %u; ecn: %u, spin: %d; "
-            "path: %hhu",
-            packet_in->pi_packno, lsquic_hety2str[packet_in->pi_header_type],
-            (unsigned) (packet_in->pi_data_sz + IQUIC_TAG_LEN),
-            lsquic_packet_in_ecn(packet_in),
-            /* spin bit value is only valid for short packet headers */
-            lsquic_packet_in_spin_bit(packet_in), packet_in->pi_path_id);
+        if (packet_in->pi_flags & PI_LOG_QL_BITS)
+            LCID("packet in: %"PRIu64", type: %s, size: %u; ecn: %u, spin: %d; "
+                "path: %hhu; Q: %d; L: %d",
+                packet_in->pi_packno, lsquic_hety2str[packet_in->pi_header_type],
+                (unsigned) (packet_in->pi_data_sz + IQUIC_TAG_LEN),
+                lsquic_packet_in_ecn(packet_in),
+                /* spin bit value is only valid for short packet headers */
+                lsquic_packet_in_spin_bit(packet_in), packet_in->pi_path_id,
+                ((packet_in->pi_flags & PI_SQUARE_BIT) > 0),
+                ((packet_in->pi_flags & PI_LOSS_BIT) > 0));
+        else
+            LCID("packet in: %"PRIu64", type: %s, size: %u; ecn: %u, spin: %d; "
+                "path: %hhu",
+                packet_in->pi_packno, lsquic_hety2str[packet_in->pi_header_type],
+                (unsigned) (packet_in->pi_data_sz + IQUIC_TAG_LEN),
+                lsquic_packet_in_ecn(packet_in),
+                /* spin bit value is only valid for short packet headers */
+                lsquic_packet_in_spin_bit(packet_in), packet_in->pi_path_id);
         break;
     }
 }
@@ -82,14 +93,10 @@ void
 lsquic_ev_log_ack_frame_in (const lsquic_cid_t *cid,
                                         const struct ack_info *acki)
 {
-    size_t sz;
-    char *buf;
+    char buf[MAX_ACKI_STR_SZ];
 
-    if ((buf = acki2str(acki, &sz)))
-    {
-        LCID("ACK frame in: %.*s", (int) sz, buf);
-        free(buf);
-    }
+    lsquic_acki2str(acki, buf, sizeof(buf));
+    LCID("ACK frame in: %s", buf);
 }
 
 
@@ -220,9 +227,10 @@ lsquic_ev_log_packet_sent (const lsquic_cid_t *cid,
                  */
                 lsquic_frame_types_to_str(frames, sizeof(frames),
                                                 packet_out->po_frame_types));
-    else
+    else if (packet_out->po_lflags & POL_LOG_QL_BITS)
         LCID("sent packet %"PRIu64", type %s, crypto: %s, size %hu, frame "
-            "types: %s, ecn: %u, spin: %d; kp: %u, path: %hhu",
+            "types: %s, ecn: %u, spin: %d; kp: %u, path: %hhu, flags: %u; "
+            "Q: %u; L: %u",
             packet_out->po_packno, lsquic_hety2str[packet_out->po_header_type],
             lsquic_enclev2str[ lsquic_packet_out_enc_level(packet_out) ],
             packet_out->po_enc_data_sz,
@@ -236,7 +244,28 @@ lsquic_ev_log_packet_sent (const lsquic_cid_t *cid,
                 /* spin bit value is only valid for short packet headers */
                 lsquic_packet_out_spin_bit(packet_out),
                 lsquic_packet_out_kp(packet_out),
-                packet_out->po_path->np_path_id);
+                packet_out->po_path->np_path_id,
+                (unsigned) packet_out->po_flags,
+                lsquic_packet_out_square_bit(packet_out),
+                lsquic_packet_out_loss_bit(packet_out));
+    else
+        LCID("sent packet %"PRIu64", type %s, crypto: %s, size %hu, frame "
+            "types: %s, ecn: %u, spin: %d; kp: %u, path: %hhu, flags: %u",
+            packet_out->po_packno, lsquic_hety2str[packet_out->po_header_type],
+            lsquic_enclev2str[ lsquic_packet_out_enc_level(packet_out) ],
+            packet_out->po_enc_data_sz,
+                /* Frame types is a list of different frames types contained
+                 * in the packet, no more.  Count and order of frames is not
+                 * printed.
+                 */
+                lsquic_frame_types_to_str(frames, sizeof(frames),
+                                                packet_out->po_frame_types),
+                lsquic_packet_out_ecn(packet_out),
+                /* spin bit value is only valid for short packet headers */
+                lsquic_packet_out_spin_bit(packet_out),
+                lsquic_packet_out_kp(packet_out),
+                packet_out->po_path->np_path_id,
+                (unsigned) packet_out->po_flags);
 }
 
 
@@ -328,9 +357,8 @@ lsquic_ev_log_generated_ack_frame (const lsquic_cid_t *cid,
                 size_t ack_buf_sz)
 {
     struct ack_info acki;
-    size_t sz;
-    char *buf;
     int len;
+    char buf[MAX_ACKI_STR_SZ];
 
     len = pf->pf_parse_ack_frame(ack_buf, ack_buf_sz, &acki,
                                                     TP_DEF_ACK_DELAY_EXP);
@@ -340,11 +368,8 @@ lsquic_ev_log_generated_ack_frame (const lsquic_cid_t *cid,
         return;
     }
 
-    if ((buf = acki2str(&acki, &sz)))
-    {
-        LCID("generated ACK frame: %.*s", (int) sz, buf);
-        free(buf);
-    }
+    lsquic_acki2str(&acki, buf, sizeof(buf));
+    LCID("generated ACK frame: %s", buf);
 }
 
 
