@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_mini_conn.c -- Mini connection.
  *
@@ -35,6 +35,7 @@
 #include "lsquic_malo.h"
 #include "lsquic_packet_common.h"
 #include "lsquic_packet_gquic.h"
+#include "lsquic_packet_ietf.h"
 #include "lsquic_packet_in.h"
 #include "lsquic_packet_out.h"
 #include "lsquic_util.h"
@@ -45,11 +46,12 @@
 #include "lsquic_sfcw.h"
 #include "lsquic_varint.h"
 #include "lsquic_hq.h"
-#include "lsquic_varint.h"
 #include "lsquic_stream.h"
 #include "lsquic_rechist.h"
 #include "lsquic_ev_log.h"
 #include "lsquic_qtags.h"
+#include "lsquic_attq.h"
+#include "lsquic_alarmset.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_MINI_CONN
 #define LSQUIC_LOG_CONN_ID lsquic_conn_log_cid(&mc->mc_conn)
@@ -57,9 +59,7 @@
 
 
 static const struct conn_iface mini_conn_iface_standard;
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-static const struct conn_iface mini_conn_iface_disable_handshake;
-#endif
+static const struct conn_iface mini_conn_iface_standard_Q050;
 
 #if LSQUIC_KEEP_MINICONN_HISTORY
 
@@ -136,6 +136,14 @@ lowest_bit_set (unsigned v)
 }
 
 
+static int
+is_handshake_stream_id (const struct mini_conn *conn,
+                                                lsquic_stream_id_t stream_id)
+{
+    return conn->mc_conn.cn_version < LSQVER_050 && stream_id == 1;
+}
+
+
 static void
 mini_destroy_packet (struct mini_conn *mc, struct lsquic_packet_out *packet_out)
 {
@@ -145,18 +153,29 @@ mini_destroy_packet (struct mini_conn *mc, struct lsquic_packet_out *packet_out)
 
 
 static int
-packet_in_is_ok (const struct lsquic_packet_in *packet_in)
+packet_in_is_ok (enum lsquic_version version,
+                                    const struct lsquic_packet_in *packet_in)
 {
+    size_t min_size;
+
     if (packet_in->pi_data_sz > GQUIC_MAX_PACKET_SZ)
     {
         LSQ_LOG1(LSQ_LOG_DEBUG, "incoming packet too large: %hu bytes",
                                                     packet_in->pi_data_sz);
         return 0;
     }
-    if (packet_in->pi_data_sz < 200)    /* TODO: 64 packets now */
-    {   /* mini connection is limited to 32 packets, yet it must send out REJ
-         * and SHLO, which are about 4 KB combined.
+
+    if ((1 << version) & LSQUIC_GQUIC_HEADER_VERSIONS)
+        /* This is a very lax number, it allows the server to send
+         * 64 * 200 = 12KB of output (REJ and SHLO).
          */
+        min_size = 200;
+    else
+        /* Chrome enforces 1200-byte minimum initial packet limit */
+        min_size = IQUIC_MIN_INIT_PACKET_SZ;
+
+    if (packet_in->pi_data_sz < min_size)
+    {
         LSQ_LOG1(LSQ_LOG_DEBUG, "incoming packet too small: %hu bytes",
                                                     packet_in->pi_data_sz);
         return 0;
@@ -173,15 +192,16 @@ mini_conn_new (struct lsquic_engine_public *enp,
     struct mini_conn *mc;
     const struct conn_iface *conn_iface;
 
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-    if (getenv("LSQUIC_DISABLE_HANDSHAKE"))
-        conn_iface = &mini_conn_iface_disable_handshake;
-    else
-#endif
+    if (!packet_in_is_ok(version, packet_in))
+        return NULL;
+    switch (version)
     {
-        if (!packet_in_is_ok(packet_in))
-            return NULL;
+    case LSQVER_050:
+        conn_iface = &mini_conn_iface_standard_Q050;
+        break;
+    default:
         conn_iface = &mini_conn_iface_standard;
+        break;
     }
 
     mc = lsquic_malo_get(enp->enp_mm.malo.mini_conn);
@@ -198,12 +218,7 @@ mini_conn_new (struct lsquic_engine_public *enp,
     TAILQ_INIT(&mc->mc_packets_out);
     mc->mc_enpub = enp;
     mc->mc_created = packet_in->pi_received;
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-    if (conn_iface == &mini_conn_iface_disable_handshake)
-        mc->mc_path.np_pack_size = 1370;
-    else
-#endif
-        mc->mc_path.np_pack_size = packet_in->pi_data_sz;
+    mc->mc_path.np_pack_size = packet_in->pi_data_sz;
     mc->mc_conn.cn_cces = mc->mc_cces;
     mc->mc_conn.cn_cces_mask = 1;
     mc->mc_conn.cn_n_cces = sizeof(mc->mc_cces) / sizeof(mc->mc_cces[0]);
@@ -342,6 +357,24 @@ process_blocked_frame (struct mini_conn *mc, lsquic_packet_in_t *packet_in,
 }
 
 
+static mconn_packno_set_t
+drop_packets_out (struct mini_conn *mc)
+{
+    struct lsquic_packet_out *packet_out;
+    mconn_packno_set_t in_flight = 0;
+
+    while ((packet_out = TAILQ_FIRST(&mc->mc_packets_out)))
+    {
+        TAILQ_REMOVE(&mc->mc_packets_out, packet_out, po_next);
+        if (packet_out->po_flags & PO_SENT)
+            in_flight |= MCONN_PACKET_MASK(packet_out->po_packno);
+        mini_destroy_packet(mc, packet_out);
+    }
+
+    return in_flight;
+}
+
+
 static unsigned
 process_connection_close_frame (struct mini_conn *mc,
         lsquic_packet_in_t *packet_in, const unsigned char *p, size_t len)
@@ -350,6 +383,8 @@ process_connection_close_frame (struct mini_conn *mc,
     uint16_t reason_len;
     uint8_t reason_off;
     int parsed_len;
+
+    (void) drop_packets_out(mc);
     parsed_len = mc->mc_conn.cn_pf->pf_parse_connect_close_frame(p, len,
                             NULL, &error_code, &reason_len, &reason_off);
     if (parsed_len < 0)
@@ -447,7 +482,7 @@ process_rst_stream_frame (struct mini_conn *mc, lsquic_packet_in_t *packet_in,
                                                                 error_code);
     LSQ_DEBUG("Got RST_STREAM; stream: %"PRIu64"; offset: 0x%"PRIX64, stream_id,
                                                                     offset);
-    if (LSQUIC_GQUIC_STREAM_HANDSHAKE == stream_id)
+    if (is_handshake_stream_id(mc, stream_id))
     {
         LSQ_INFO("handshake stream reset, closing connection");
         return 0;
@@ -491,7 +526,7 @@ process_stream_frame (struct mini_conn *mc, lsquic_packet_in_t *packet_in,
         return 0;
     EV_LOG_STREAM_FRAME_IN(LSQUIC_LOG_CONN_ID, &stream_frame);
     LSQ_DEBUG("Got stream frame for stream #%"PRIu64, stream_frame.stream_id);
-    if (LSQUIC_GQUIC_STREAM_HANDSHAKE == stream_frame.stream_id)
+    if (is_handshake_stream_id(mc, stream_frame.stream_id))
     {
         if (packet_in->pi_flags & PI_HSK_STREAM)
         {   /* This is not supported for simplicity.  The spec recommends
@@ -510,8 +545,12 @@ process_stream_frame (struct mini_conn *mc, lsquic_packet_in_t *packet_in,
             mc->mc_flags |= MC_HAVE_NEW_HSK;
             MCHIST_APPEND(mc, MCHE_NEW_HSK);
             if (0 == stream_frame.data_frame.df_offset)
+            {
                 /* First CHLO message: update maximum packet size */
                 mc->mc_path.np_pack_size = packet_in->pi_data_sz;
+                LSQ_DEBUG("update packet size to %hu",
+                                                    mc->mc_path.np_pack_size);
+            }
         }
         else
         {
@@ -523,6 +562,54 @@ process_stream_frame (struct mini_conn *mc, lsquic_packet_in_t *packet_in,
 }
 
 
+static unsigned
+process_crypto_frame (struct mini_conn *mc, struct lsquic_packet_in *packet_in,
+                                          const unsigned char *p, size_t len)
+{
+    stream_frame_t stream_frame;
+    int parsed_len;
+    parsed_len = mc->mc_conn.cn_pf->pf_parse_crypto_frame(p, len,
+                                                                &stream_frame);
+    if (parsed_len < 0)
+        return 0;
+    EV_LOG_CRYPTO_FRAME_IN(LSQUIC_LOG_CONN_ID, &stream_frame,
+                                        lsquic_packet_in_enc_level(packet_in));
+    LSQ_DEBUG("Got CRYPTO frame at encryption level %s",
+                    lsquic_enclev2str[lsquic_packet_in_enc_level(packet_in)]);
+    if (packet_in->pi_flags & PI_HSK_STREAM)
+    {   /* This is not supported for simplicity: assume a single CRYPTO frame
+         * per packet.  If this changes, we can revisit this code.
+         */
+        LSQ_INFO("two CRYPTO frames in single incoming packet");
+        MCHIST_APPEND(mc, MCHE_2HSK_1STREAM);
+        return 0;
+    }
+    if (stream_frame.data_frame.df_offset >= mc->mc_read_off)
+    {
+        packet_in->pi_flags |= PI_HSK_STREAM;
+        packet_in->pi_hsk_stream = p - packet_in->pi_data;
+        mc->mc_flags |= MC_HAVE_NEW_HSK;
+        MCHIST_APPEND(mc, MCHE_NEW_HSK);
+        if (0 == stream_frame.data_frame.df_offset)
+        {
+            /* First CHLO message: update maximum packet size */
+            mc->mc_path.np_pack_size = packet_in->pi_data_sz
+                /* Q050 and later adjust pi_data_sz of Initial packets during
+                 * decryption, here we have to add the tag length back:
+                 */
+                                        + mc->mc_conn.cn_esf_c->esf_tag_len;
+            LSQ_DEBUG("update packet size to %hu", mc->mc_path.np_pack_size);
+        }
+    }
+    else
+    {
+        LSQ_DEBUG("drop duplicate frame");
+        MCHIST_APPEND(mc, MCHE_DUP_HSK);
+    }
+    return parsed_len;
+}
+
+
 static unsigned
 process_window_update_frame (struct mini_conn *mc,
             lsquic_packet_in_t *packet_in, const unsigned char *p, size_t len)
@@ -534,7 +621,7 @@ process_window_update_frame (struct mini_conn *mc,
     if (parsed_len < 0)
         return 0;
     EV_LOG_WINDOW_UPDATE_FRAME_IN(LSQUIC_LOG_CONN_ID, stream_id, offset);
-    if (LSQUIC_GQUIC_STREAM_HANDSHAKE == stream_id)
+    if (is_handshake_stream_id(mc, stream_id))
         /* This should not happen: why would the client send us WINDOW_UPDATE
          * on stream 1?
          */
@@ -553,6 +640,7 @@ static process_frame_f const process_frames[N_QUIC_FRAMES] =
     [QUIC_FRAME_ACK]                  =  process_ack_frame,
     [QUIC_FRAME_BLOCKED]              =  process_blocked_frame,
     [QUIC_FRAME_CONNECTION_CLOSE]     =  process_connection_close_frame,
+    [QUIC_FRAME_CRYPTO]               =  process_crypto_frame,
     [QUIC_FRAME_GOAWAY]               =  process_goaway_frame,
     [QUIC_FRAME_INVALID]              =  process_invalid_frame,
     [QUIC_FRAME_PADDING]              =  process_padding_frame,
@@ -568,7 +656,7 @@ static unsigned
 process_packet_frame (struct mini_conn *mc, lsquic_packet_in_t *packet_in,
                       const unsigned char *p, size_t len)
 {
-    enum quic_frame_type type = mc->mc_conn.cn_pf->pf_parse_frame_type(p[0]);
+    enum quic_frame_type type = mc->mc_conn.cn_pf->pf_parse_frame_type(p, len);
     packet_in->pi_frame_types |= 1 << type;
     return process_frames[type](mc, packet_in, p, len);
 }
@@ -594,56 +682,64 @@ record_largest_recv (struct mini_conn *mc, lsquic_time_t t)
 static enum dec_packin
 conn_decrypt_packet (struct mini_conn *conn, lsquic_packet_in_t *packet_in)
 {
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-    if (getenv("LSQUIC_DISABLE_HANDSHAKE"))
-    {
-        (void) lsquic_conn_copy_and_release_pi_data(&conn->mc_conn,
-                                                conn->mc_enpub, packet_in);
-        packet_in->pi_flags |= PI_DECRYPTED;
-        return 0;
-    }
-    else
-#endif
-        return conn->mc_conn.cn_esf_c->esf_decrypt_packet(
+    return conn->mc_conn.cn_esf_c->esf_decrypt_packet(
                         conn->mc_conn.cn_enc_session, conn->mc_enpub,
                         &conn->mc_conn, packet_in);
 }
 
 
-static enum { PRP_KEEP, PRP_DEFER, PRP_DROP, PRP_ERROR }
+/* PRP: Process Regular Packet */
+enum proc_rp { PRP_KEEP, PRP_DEFER, PRP_DROP, PRP_ERROR, };
+
+
+static enum proc_rp
+conn_decrypt_packet_or (struct mini_conn *mc,
+                                        struct lsquic_packet_in *packet_in)
+{
+    if (DECPI_OK == conn_decrypt_packet(mc, packet_in))
+    {
+        MCHIST_APPEND(mc, MCHE_DECRYPTED);
+        return PRP_KEEP;
+    }
+    else if (mc->mc_conn.cn_esf.g->esf_have_key_gt_one(
+                                            mc->mc_conn.cn_enc_session))
+    {
+        LSQ_INFO("could not decrypt packet: drop");
+        mc->mc_dropped_packnos |= MCONN_PACKET_MASK(packet_in->pi_packno);
+        MCHIST_APPEND(mc, MCHE_UNDECR_DROP);
+        return PRP_DROP;
+    }
+    else if ((packet_in->pi_flags & PI_OWN_DATA) ||
+            0 == lsquic_conn_copy_and_release_pi_data(&mc->mc_conn,
+                                                mc->mc_enpub, packet_in))
+    {
+        assert(packet_in->pi_flags & PI_OWN_DATA);
+        LSQ_INFO("could not decrypt packet: defer");
+        mc->mc_deferred_packnos |= MCONN_PACKET_MASK(packet_in->pi_packno);
+        MCHIST_APPEND(mc, MCHE_UNDECR_DEFER);
+        return PRP_DEFER;
+    }
+    else
+    {
+        MCHIST_APPEND(mc, MCHE_ENOMEM);
+        return PRP_ERROR;   /* Memory allocation must have failed */
+    }
+}
+
+
+static enum proc_rp
 process_regular_packet (struct mini_conn *mc, lsquic_packet_in_t *packet_in)
 {
     const unsigned char *p, *pend;
+    enum proc_rp prp;
     unsigned len;
 
     /* Decrypt packet if necessary */
     if (0 == (packet_in->pi_flags & PI_DECRYPTED))
     {
-        if (DECPI_OK == conn_decrypt_packet(mc, packet_in))
-            MCHIST_APPEND(mc, MCHE_DECRYPTED);
-        else if (mc->mc_conn.cn_esf.g->esf_have_key_gt_one(
-                                                mc->mc_conn.cn_enc_session))
-        {
-            LSQ_INFO("could not decrypt packet: drop");
-            mc->mc_dropped_packnos |= MCONN_PACKET_MASK(packet_in->pi_packno);
-            MCHIST_APPEND(mc, MCHE_UNDECR_DROP);
-            return PRP_DROP;
-        }
-        else if ((packet_in->pi_flags & PI_OWN_DATA) ||
-                0 == lsquic_conn_copy_and_release_pi_data(&mc->mc_conn,
-                                                    mc->mc_enpub, packet_in))
-        {
-            assert(packet_in->pi_flags & PI_OWN_DATA);
-            LSQ_INFO("could not decrypt packet: defer");
-            mc->mc_deferred_packnos |= MCONN_PACKET_MASK(packet_in->pi_packno);
-            MCHIST_APPEND(mc, MCHE_UNDECR_DEFER);
-            return PRP_DEFER;
-        }
-        else
-        {
-            MCHIST_APPEND(mc, MCHE_ENOMEM);
-            return PRP_ERROR;   /* Memory allocation must have failed */
-        }
+        prp = conn_decrypt_packet_or(mc, packet_in);
+        if (prp != PRP_KEEP)
+            return prp;
     }
 
     /* Update receive history before processing the packet: if there is an
@@ -665,7 +761,7 @@ process_regular_packet (struct mini_conn *mc, lsquic_packet_in_t *packet_in)
             p += len;
         else
         {
-            if (mc->mc_conn.cn_pf->pf_parse_frame_type(p[0]) !=
+            if (mc->mc_conn.cn_pf->pf_parse_frame_type(p, pend - p) !=
                                                     QUIC_FRAME_CONNECTION_CLOSE)
                 LSQ_WARN("error parsing frame: packno %"PRIu64"; sz: %u; type: "
                     "0x%X", packet_in->pi_packno, packet_in->pi_data_sz, p[0]);
@@ -727,6 +823,18 @@ mini_stream_read (void *stream, void *buf, size_t len, int *reached_fin)
 }
 
 
+/* Wrapper to throw out reached_fin */
+static size_t
+mini_stream_read_for_crypto (void *stream, void *buf, size_t len)
+{
+    size_t retval;
+    int reached_fin;
+
+    retval = mini_stream_read(stream, buf, len, &reached_fin);
+    return retval;
+}
+
+
 static size_t
 mini_stream_size (void *stream)
 {
@@ -789,6 +897,13 @@ allocate_packet_out (struct mini_conn *mc, const unsigned char *nonce)
         packet_out->po_flags |= PO_HELLO;
         packet_out->po_header_type = HETY_0RTT;
     }
+    if (mc->mc_conn.cn_version >= LSQVER_050)
+    {
+        if (nonce)
+            packet_out->po_header_type = HETY_0RTT;
+        else
+            packet_out->po_header_type = HETY_INITIAL;
+    }
     lsquic_packet_out_set_pns(packet_out, PNS_APP);
     TAILQ_INSERT_TAIL(&mc->mc_packets_out, packet_out, po_next);
     LSQ_DEBUG("allocated packet #%"PRIu64", nonce: %d", packno, !!nonce);
@@ -798,14 +913,89 @@ allocate_packet_out (struct mini_conn *mc, const unsigned char *nonce)
 }
 
 
+static struct lsquic_packet_out *
+to_packet_pre_Q050 (struct mini_conn *mc, struct mini_stream_ctx *ms_ctx,
+                    const unsigned char *nonce)
+{
+    struct lsquic_packet_out *packet_out;
+    size_t cur_off;
+    int len;
+
+    packet_out = allocate_packet_out(mc, nonce);
+    if (!packet_out)
+        return NULL;
+    cur_off = ms_ctx->off;
+    len = mc->mc_conn.cn_pf->pf_gen_stream_frame(
+            packet_out->po_data + packet_out->po_data_sz,
+            lsquic_packet_out_avail(packet_out),
+            1, mc->mc_write_off, mini_stream_fin(ms_ctx),
+            mini_stream_size(ms_ctx), mini_stream_read, ms_ctx);
+    if (len < 0)
+    {
+        LSQ_WARN("cannot generate STREAM frame (avail: %u)",
+                                    lsquic_packet_out_avail(packet_out));
+        return NULL;
+    }
+    mc->mc_write_off += ms_ctx->off - cur_off;
+    EV_LOG_GENERATED_STREAM_FRAME(LSQUIC_LOG_CONN_ID, mc->mc_conn.cn_pf,
+                        packet_out->po_data + packet_out->po_data_sz, len);
+    packet_out->po_data_sz += len;
+    packet_out->po_frame_types |= 1 << QUIC_FRAME_STREAM;
+    if (0 == lsquic_packet_out_avail(packet_out))
+        packet_out->po_flags |= PO_STREAM_END;
+
+    return packet_out;
+}
+
+
+static struct lsquic_packet_out *
+to_packet_Q050plus (struct mini_conn *mc, struct mini_stream_ctx *ms_ctx,
+                    const unsigned char *nonce)
+{
+    struct lsquic_packet_out *packet_out;
+    size_t cur_off;
+    int len;
+
+    if (nonce && !(mc->mc_flags & MC_WR_OFF_RESET))
+    {
+        mc->mc_write_off = 0;
+        mc->mc_flags |= MC_WR_OFF_RESET;
+    }
+
+    packet_out = allocate_packet_out(mc, nonce);
+    if (!packet_out)
+        return NULL;
+    cur_off = ms_ctx->off;
+    len = mc->mc_conn.cn_pf->pf_gen_crypto_frame(
+            packet_out->po_data + packet_out->po_data_sz,
+            lsquic_packet_out_avail(packet_out), mc->mc_write_off,
+            mini_stream_size(ms_ctx), mini_stream_read_for_crypto, ms_ctx);
+    if (len < 0)
+    {
+        LSQ_WARN("cannot generate CRYPTO frame (avail: %u)",
+                                    lsquic_packet_out_avail(packet_out));
+        return NULL;
+    }
+    mc->mc_write_off += ms_ctx->off - cur_off;
+    EV_LOG_GENERATED_CRYPTO_FRAME(LSQUIC_LOG_CONN_ID, mc->mc_conn.cn_pf,
+                        packet_out->po_data + packet_out->po_data_sz, len);
+    packet_out->po_data_sz += len;
+    packet_out->po_frame_types |= 1 << QUIC_FRAME_CRYPTO;
+
+    return packet_out;
+}
+
+
 static int
 packetize_response (struct mini_conn *mc, const unsigned char *buf,
                     size_t bufsz, const unsigned char *nonce)
 {
     struct mini_stream_ctx ms_ctx;
     lsquic_packet_out_t *packet_out;
-    size_t cur_off;
-    int len;
+    struct lsquic_packet_out * (*const to_packet) (struct mini_conn *,
+                struct mini_stream_ctx *, const unsigned char *)
+        = mc->mc_conn.cn_version < LSQVER_050
+            ? to_packet_pre_Q050 : to_packet_Q050plus;
 
     LSQ_DEBUG("Packetizing %zd bytes of handshake response", bufsz);
 
@@ -815,27 +1005,9 @@ packetize_response (struct mini_conn *mc, const unsigned char *buf,
 
     do
     {
-        packet_out = allocate_packet_out(mc, nonce);
+        packet_out = to_packet(mc, &ms_ctx, nonce);
         if (!packet_out)
             return -1;
-        cur_off = ms_ctx.off;
-        len = mc->mc_conn.cn_pf->pf_gen_stream_frame(packet_out->po_data + packet_out->po_data_sz,
-                    lsquic_packet_out_avail(packet_out),
-                    LSQUIC_GQUIC_STREAM_HANDSHAKE, mc->mc_write_off, mini_stream_fin(&ms_ctx),
-                    mini_stream_size(&ms_ctx), mini_stream_read, &ms_ctx);
-        if (len < 0)
-        {
-            LSQ_WARN("cannot generate stream frame (avail: %u)",
-                                        lsquic_packet_out_avail(packet_out));
-            return -1;
-        }
-        mc->mc_write_off += ms_ctx.off - cur_off;
-        EV_LOG_GENERATED_STREAM_FRAME(LSQUIC_LOG_CONN_ID, mc->mc_conn.cn_pf,
-                            packet_out->po_data + packet_out->po_data_sz, len);
-        packet_out->po_data_sz += len;
-        packet_out->po_frame_types |= 1 << QUIC_FRAME_STREAM;
-        if (0 == lsquic_packet_out_avail(packet_out))
-            packet_out->po_flags |= PO_STREAM_END;
     }
     while (mini_stream_has_data(&ms_ctx));
 
@@ -871,6 +1043,10 @@ continue_handshake (struct mini_conn *mc)
     struct hsk_chunk hsk_chunks[MINICONN_MAX_PACKETS], *hsk_chunk;
     unsigned char nonce_buf[32];
     int nonce_set = 0;
+    int (*parse_frame)(const unsigned char *, size_t, struct stream_frame *)
+        = mc->mc_conn.cn_version < LSQVER_050
+            ? mc->mc_conn.cn_pf->pf_parse_stream_frame
+            : mc->mc_conn.cn_pf->pf_parse_crypto_frame;
 
     /* Get handshake stream data from each packet that contains a handshake
      * stream frame and place them into `hsk_chunks' array.
@@ -880,7 +1056,7 @@ continue_handshake (struct mini_conn *mc)
         assert(n_hsk_chunks < sizeof(hsk_chunks) / sizeof(hsk_chunks[0]));
         if (0 == (packet_in->pi_flags & PI_HSK_STREAM))
             continue;
-        s = mc->mc_conn.cn_pf->pf_parse_stream_frame(packet_in->pi_data + packet_in->pi_hsk_stream,
+        s = parse_frame(packet_in->pi_data + packet_in->pi_hsk_stream,
                 packet_in->pi_data_sz - packet_in->pi_hsk_stream, &frame);
         if (-1 == s)
         {
@@ -983,8 +1159,8 @@ continue_handshake (struct mini_conn *mc)
     if (!mc->mc_conn.cn_enc_session)
     {
         mc->mc_conn.cn_enc_session =
-            mc->mc_conn.cn_esf.g->esf_create_server(mc->mc_conn.cn_cid,
-                                                    mc->mc_enpub);
+            mc->mc_conn.cn_esf.g->esf_create_server(&mc->mc_conn,
+                                        mc->mc_conn.cn_cid, mc->mc_enpub);
         if (!mc->mc_conn.cn_enc_session)
         {
             LSQ_WARN("cannot create new enc session");
@@ -1463,7 +1639,14 @@ process_packet (struct mini_conn *mc, struct lsquic_packet_in *packet_in)
     case PRP_DEFER:
         assert(packet_in->pi_flags & PI_OWN_DATA);
         lsquic_packet_in_upref(packet_in);
-        TAILQ_INSERT_TAIL(&mc->mc_deferred, packet_in, pi_next);
+        if (mc->mc_n_deferred < MINI_CONN_MAX_DEFERRED)
+        {
+            TAILQ_INSERT_TAIL(&mc->mc_deferred, packet_in, pi_next);
+            ++mc->mc_n_deferred;
+        }
+        else
+            LSQ_DEBUG("won't defer more than %u packets: drop",
+                                                MINI_CONN_MAX_DEFERRED);
         break;
     case PRP_ERROR:
         mc->mc_flags |= MC_ERROR;
@@ -1492,6 +1675,7 @@ insert_into_deferred (struct mini_conn *mc, lsquic_packet_in_t *new_packet)
         TAILQ_INSERT_BEFORE(packet_in, new_packet, pi_next);
     else
         TAILQ_INSERT_TAIL(&mc->mc_deferred, new_packet, pi_next);
+    ++mc->mc_n_deferred;
 }
 
 
@@ -1506,6 +1690,7 @@ process_deferred_packets (struct mini_conn *mc)
     {
         packet_in = TAILQ_FIRST(&mc->mc_deferred);
         TAILQ_REMOVE(&mc->mc_deferred, packet_in, pi_next);
+        --mc->mc_n_deferred;
         process_packet(mc, packet_in);
         reached_last = packet_in == last;
         lsquic_packet_in_put(&mc->mc_enpub->enp_mm, packet_in);
@@ -1640,11 +1825,75 @@ mini_conn_ci_packet_in (struct lsquic_conn *lconn,
 
     if (TAILQ_EMPTY(&mc->mc_deferred))
         process_packet(mc, packet_in);
-    else
+    else if (mc->mc_n_deferred < MINI_CONN_MAX_DEFERRED)
     {
         insert_into_deferred(mc, packet_in);
         process_deferred_packets(mc);
     }
+    else
+        LSQ_DEBUG("won't defer more than %u packets: drop",
+                                                MINI_CONN_MAX_DEFERRED);
+}
+
+
+/* Q050 is different is that packet numbers are not known until after the
+ * packet is decrypted, so we have to follow different logic here.
+ */
+static void
+mini_conn_ci_Q050_packet_in (struct lsquic_conn *lconn,
+                        struct lsquic_packet_in *packet_in)
+{
+    struct mini_conn *mc = (struct mini_conn *) lconn;
+    enum proc_rp prp;
+
+    if (mc->mc_flags & MC_ERROR)
+    {
+        LSQ_DEBUG("error state: ignore packet");
+        return;
+    }
+
+    if (!mc->mc_conn.cn_enc_session)
+    {
+        mc->mc_conn.cn_enc_session =
+            mc->mc_conn.cn_esf.g->esf_create_server(&mc->mc_conn,
+                                        mc->mc_conn.cn_cid, mc->mc_enpub);
+        if (!mc->mc_conn.cn_enc_session)
+        {
+            LSQ_WARN("cannot create new enc session");
+            mc->mc_flags |= MC_ERROR;
+            return;
+        }
+        MCHIST_APPEND(mc, MCHE_NEW_ENC_SESS);
+    }
+
+    assert(!(packet_in->pi_flags & PI_DECRYPTED));
+    prp = conn_decrypt_packet_or(mc, packet_in);
+    switch (prp)
+    {
+    case PRP_KEEP:
+        break;
+    case PRP_DROP:
+        return;
+    case PRP_ERROR:
+        mc->mc_flags |= MC_ERROR;
+        return;
+    default:
+        if (mc->mc_n_deferred >= MINI_CONN_MAX_DEFERRED)
+        {
+            LSQ_DEBUG("won't defer more than %u packets: drop",
+                                                MINI_CONN_MAX_DEFERRED);
+            return;
+        }
+        assert(prp == PRP_DEFER);
+        assert(packet_in->pi_flags & PI_OWN_DATA);
+        lsquic_packet_in_upref(packet_in);
+        TAILQ_INSERT_TAIL(&mc->mc_deferred, packet_in, pi_next);
+        ++mc->mc_n_deferred;
+        return;
+    }
+
+    assert(prp == PRP_KEEP);
+    process_packet(mc, packet_in);
 }
 
 
@@ -1700,8 +1949,7 @@ mini_conn_ci_destroy (struct lsquic_conn *lconn)
     assert(!(lconn->cn_flags & LSCONN_HASHED));
     struct mini_conn *mc = (struct mini_conn *) lconn;
     lsquic_packet_in_t *packet_in;
-    lsquic_packet_out_t *packet_out;
-    mconn_packno_set_t still_deferred = 0, in_flight = 0;
+    mconn_packno_set_t still_deferred = 0, in_flight;
     enum lsq_log_level log_level;
 #if LSQUIC_RECORD_INORD_HIST
     char inord_str[0x100];
@@ -1714,16 +1962,14 @@ mini_conn_ci_destroy (struct lsquic_conn *lconn)
     while ((packet_in = TAILQ_FIRST(&mc->mc_deferred)))
     {
         TAILQ_REMOVE(&mc->mc_deferred, packet_in, pi_next);
+        --mc->mc_n_deferred;
         still_deferred |= MCONN_PACKET_MASK(packet_in->pi_packno);
         lsquic_packet_in_put(&mc->mc_enpub->enp_mm, packet_in);
     }
-    while ((packet_out = TAILQ_FIRST(&mc->mc_packets_out)))
-    {
-        TAILQ_REMOVE(&mc->mc_packets_out, packet_out, po_next);
-        if (packet_out->po_flags & PO_SENT)
-            in_flight |= MCONN_PACKET_MASK(packet_out->po_packno);
-        mini_destroy_packet(mc, packet_out);
-    }
+    if (TAILQ_EMPTY(&mc->mc_packets_out))
+        in_flight = ~0ull;  /* Indicates that packets were dropped before */
+    else
+        in_flight = drop_packets_out(mc);
     if (mc->mc_conn.cn_enc_session)
         mc->mc_conn.cn_esf.g->esf_destroy(mc->mc_conn.cn_enc_session);
     log_level = warning_is_warranted(mc) ? LSQ_LOG_WARN : LSQ_LOG_DEBUG;
@@ -1810,44 +2056,6 @@ mini_conn_ci_destroy (struct lsquic_conn *lconn)
 }
 
 
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-static void
-mini_conn_ci_packet_in_disable_handshake (struct lsquic_conn *lconn,
-                                          struct lsquic_packet_in *packet_in)
-{
-    struct mini_conn *mc = (struct mini_conn *) lconn;
-    if (0 == (mc->mc_flags & MC_ERROR))
-    {
-        if (packet_in->pi_packno > MINICONN_MAX_PACKETS)
-            mc->mc_flags |= MC_ERROR;
-        else if (!(mc->mc_received_packnos &
-                                    MCONN_PACKET_MASK(packet_in->pi_packno)))
-        {
-            lsquic_packet_in_upref(packet_in);
-            TAILQ_INSERT_TAIL(&mc->mc_packets_in, packet_in, pi_next);
-            mc->mc_received_packnos |= MCONN_PACKET_MASK(packet_in->pi_packno);
-        }
-    }
-}
-
-
-static enum tick_st
-mini_conn_ci_tick_disable_handshake (struct lsquic_conn *lconn,
-                                     lsquic_time_t now)
-{
-    struct mini_conn *mc = (struct mini_conn *) lconn;
-    LSQ_WARN("handshake disabled, promote me now");
-    lconn->cn_flags |= LSCONN_HANDSHAKE_DONE;
-    mc->mc_conn.cn_enc_session =
-        mc->mc_conn.cn_esf.g->esf_create_server(lconn->cn_cid, mc->mc_enpub, 0);
-    mc->mc_conn.cn_esf.g->esf_set_handshake_completed(mc->mc_conn.cn_enc_session);
-    return TICK_PROMOTE;
-}
-
-
-#endif
-
-
 static struct lsquic_engine *
 mini_conn_ci_get_engine (struct lsquic_conn *lconn)
 {
@@ -1863,20 +2071,30 @@ mini_conn_ci_hsk_done (struct lsquic_conn *lconn, enum lsquic_hsk_status status)
 }
 
 
+/* A mini connection is only tickable if it has unsent packets.  This can
+ * occur when packet sending is delayed.
+ *
+ * Otherwise, a mini connection is not tickable:  Either there are incoming
+ * packets, in which case, the connection is going to be ticked, or there is
+ * an alarm pending, in which case it will be handled via the attq.
+ */
 static int
 mini_conn_ci_is_tickable (struct lsquic_conn *lconn)
 {
-    /* A mini connection is never tickable:  Either there are incoming
-     * packets, in which case, the connection is going to be ticked, or
-     * there is an alarm pending, in which case it will be handled via
-     * the attq.
-     */
+    struct mini_conn *const mc = (struct mini_conn *) lconn;
+    const struct lsquic_packet_out *packet_out;
+
+    if (mc->mc_enpub->enp_flags & ENPUB_CAN_SEND)
+        TAILQ_FOREACH(packet_out, &mc->mc_packets_out, po_next)
+            if (!(packet_out->po_flags & PO_SENT))
+                return 1;
+
     return 0;
 }
 
 
 static lsquic_time_t
-mini_conn_ci_next_tick_time (struct lsquic_conn *lconn)
+mini_conn_ci_next_tick_time (struct lsquic_conn *lconn, unsigned *why)
 {
     struct mini_conn *mc = (struct mini_conn *) lconn;
     lsquic_packet_out_t *packet_out;
@@ -1889,11 +2107,18 @@ mini_conn_ci_next_tick_time (struct lsquic_conn *lconn)
         {
             retx_time = packet_out->po_sent + calc_retx_timeout(mc);
             if (retx_time < exp_time)
+            {
+                *why = N_AEWS + AL_RETX_HSK;
                 return retx_time;
+            }
             else
+            {
+                *why = AEW_MINI_EXPIRE;
                 return exp_time;
+            }
         }
 
+    *why = AEW_MINI_EXPIRE;
     return exp_time;
 }
 
@@ -1987,24 +2212,25 @@ static const struct conn_iface mini_conn_iface_standard = {
 };
 
 
-#if LSQUIC_ENABLE_HANDSHAKE_DISABLE
-static const struct conn_iface mini_conn_iface_disable_handshake = {
+static const struct conn_iface mini_conn_iface_standard_Q050 = {
     .ci_abort_error          =  mini_conn_ci_abort_error,
     .ci_client_call_on_new   =  mini_conn_ci_client_call_on_new,
     .ci_destroy              =  mini_conn_ci_destroy,
     .ci_get_engine           =  mini_conn_ci_get_engine,
+    .ci_get_path             =  mini_conn_ci_get_path,
     .ci_hsk_done             =  mini_conn_ci_hsk_done,
     .ci_internal_error       =  mini_conn_ci_internal_error,
     .ci_is_tickable          =  mini_conn_ci_is_tickable,
     .ci_next_packet_to_send  =  mini_conn_ci_next_packet_to_send,
     .ci_next_tick_time       =  mini_conn_ci_next_tick_time,
-    .ci_packet_in            =  mini_conn_ci_packet_in_disable_handshake,
+    .ci_packet_in            =  mini_conn_ci_Q050_packet_in,
     .ci_packet_not_sent      =  mini_conn_ci_packet_not_sent,
     .ci_packet_sent          =  mini_conn_ci_packet_sent,
-    .ci_tick                 =  mini_conn_ci_tick_disable_handshake,
+    .ci_record_addrs         =  mini_conn_ci_record_addrs,
+    .ci_tick                 =  mini_conn_ci_tick,
     .ci_tls_alert            =  mini_conn_ci_tls_alert,
 };
-#endif
+
 
 typedef char largest_recv_holds_at_least_16_seconds[
     ((1 << (sizeof(((struct mini_conn *) 0)->mc_largest_recv) * 8)) / 1000000
