@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_mini_conn_ietf.h -- Mini connection used by the IETF QUIC
  */
@@ -54,6 +54,10 @@ struct ietf_mini_conn
         IMC_BAD_TRANS_PARAMS    = 1 << 16,
         IMC_ADDR_VALIDATED      = 1 << 17,
         IMC_HSK_PACKET_SENT     = 1 << 18,
+        IMC_CLOSE_RECVD         = 1 << 19,
+        IMC_PARSE_FAILED        = 1 << 20,
+        IMC_PATH_CHANGED        = 1 << 21,
+        IMC_HSK_DONE_SENT       = 1 << 22,
     }                               imc_flags;
     struct mini_crypto_stream       imc_streams[N_ENC_LEVS];
     void                           *imc_stream_ps[N_ENC_LEVS];
@@ -63,6 +67,7 @@ struct ietf_mini_conn
     }                               imc_last_in;
     TAILQ_HEAD(, lsquic_packet_in)  imc_app_packets;
     TAILQ_HEAD(, lsquic_packet_out) imc_packets_out;
+    TAILQ_HEAD(, stream_frame)      imc_crypto_frames;
     packno_set_t                    imc_sent_packnos;
     packno_set_t                    imc_recvd_packnos[N_PNS];
     packno_set_t                    imc_acked_packnos[N_PNS];
@@ -71,6 +76,11 @@ struct ietf_mini_conn
     unsigned                        imc_error_code;
     unsigned                        imc_bytes_in;
     unsigned                        imc_bytes_out;
+    unsigned short                  imc_crypto_frames_sz;
+    /* We need to read in the length of ClientHello to check when we have fed
+     * it to the crypto layer.
+     */
+    unsigned short                  imc_ch_len;
     unsigned char                   imc_next_packno;
     unsigned char                   imc_hsk_count;
     /* We don't send more than eight in the first flight, and so it's OK to
@@ -84,13 +94,27 @@ struct ietf_mini_conn
     uint8_t                         imc_ecn_counts_out[N_PNS][4];
     uint8_t                         imc_incoming_ecn;
     uint8_t                         imc_tls_alert;
+#define IMICO_MAX_STASHED_FRAMES 10u
+    unsigned char                   imc_n_crypto_frames;
     struct network_path             imc_path;
 };
 
+/* [draft-ietf-quic-transport-24] Section 7.4
+ *
+ " Implementations MUST support buffering at least 4096 bytes of data
+ " received in CRYPTO frames out of order.  Endpoints MAY choose to
+ " allow more data to be buffered during the handshake.  A larger limit
+ " during the handshake could allow for larger keys or credentials to be
+ " exchanged.  An endpoint's buffer size does not need to remain
+ " constant during the life of the connection.
+ */
+#define IMICO_MAX_BUFFERED_CRYPTO (6u * 1024u)
+
 struct lsquic_conn *
 lsquic_mini_conn_ietf_new (struct lsquic_engine_public *,
                const struct lsquic_packet_in *,
-               enum lsquic_version, int is_ipv4, const struct lsquic_cid *);
+               enum lsquic_version, int is_ipv4, const struct lsquic_cid *,
+               size_t udp_payload_size);
 
 int
 lsquic_mini_conn_ietf_ecn_ok (const struct ietf_mini_conn *);
