@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_packints.h -- Ordered (high to low) list of packet intervals.
  */
@@ -15,8 +15,10 @@ struct packet_interval {
     struct lsquic_packno_range      range;
 };
 
+TAILQ_HEAD(pinhead, packet_interval);
+
 struct packints {
-    TAILQ_HEAD(, packet_interval)   pk_intervals;
+    struct pinhead                  pk_intervals;
     struct packet_interval         *pk_cur;
 };
 
