@@ -1,7 +1,7 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_parse_gquic_be.c -- Parsing functions specific to big-endian
- *                              (Q039 and higher) GQUIC.
+ *                              (now only Q043) GQUIC.
  */
 
 #include <assert.h>
@@ -375,6 +375,7 @@ parse_ack_frame_without_blocks (const unsigned char *buf, size_t buf_len,
     const unsigned char type = buf[0];
     const unsigned char *p = buf + 1;
     const unsigned char *const pend = buf + buf_len;
+    unsigned char n_timestamps;
 
     const int ack_block_len   = twobit_to_1246(type & 3);        /* mm */
     const int largest_obs_len = twobit_to_1246((type >> 2) & 3); /* ll */
@@ -393,12 +394,12 @@ parse_ack_frame_without_blocks (const unsigned char *buf, size_t buf_len,
 
     ack->n_ranges = 1;
 
-    ack->n_timestamps = *p;
+    n_timestamps = *p;
     ++p;
 
-    if (ack->n_timestamps)
+    if (n_timestamps)
     {
-        unsigned timestamps_size = 5 + 3 * (ack->n_timestamps - 1);
+        unsigned timestamps_size = 5 + 3 * (n_timestamps - 1);
         CHECK_SPACE(timestamps_size, p, pend);
         p += timestamps_size;
     }
@@ -419,6 +420,7 @@ parse_ack_frame_with_blocks (const unsigned char *buf, size_t buf_len,
     const unsigned char type = buf[0];
     const unsigned char *p = buf + 1;
     const unsigned char *const pend = buf + buf_len;
+    unsigned char n_timestamps;
 
     assert((type & 0xC0) == 0x40);      /* We're passed correct frame type */
 
@@ -461,31 +463,14 @@ parse_ack_frame_with_blocks (const unsigned char *buf, size_t buf_len,
     }
     ack->n_ranges = n;
 
-    ack->n_timestamps = *p;
+    n_timestamps = *p;
     ++p;
 
-    if (ack->n_timestamps)
+    if (n_timestamps)
     {
-#if LSQUIC_PARSE_ACK_TIMESTAMPS
-        CHECK_SPACE(5, p , pend);
-        ack->timestamps[0].packet_delta = *p++;
-        memcpy(&ack->timestamps[0].delta_usec, p, 4);
-        p += 4;
-        unsigned i;
-        for (i = 1; i < ack->n_timestamps; ++i)
-        {
-            CHECK_SPACE(3, p , pend);
-            ack->timestamps[i].packet_delta = *p++;
-            uint64_t delta_time = read_float_time16(p);
-            p += 2;
-            ack->timestamps[i].delta_usec =
-                ack->timestamps[i - 1].delta_usec + delta_time;
-        }
-#else
-        unsigned timestamps_size = 5 + 3 * (ack->n_timestamps - 1);
+        unsigned timestamps_size = 5 + 3 * (n_timestamps - 1);
         CHECK_SPACE(timestamps_size, p, pend);
         p += timestamps_size;
-#endif
     }
 
     assert(p <= pend);
@@ -703,6 +688,14 @@ gquic_be_gen_ping_frame (unsigned char *buf, int buf_len)
 }
 
 
+size_t
+gquic_be_connect_close_frame_size (int app_error, unsigned error_code,
+                                unsigned frame_type, size_t reason_len)
+{
+    return 1 + 4 + 2 + reason_len;
+}
+
+
 int
 gquic_be_gen_connect_close_frame (unsigned char *buf, size_t buf_len,
     int app_error_UNUSED, unsigned ecode, const char *reason, int reason_len)
@@ -894,9 +887,10 @@ gquic_be_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
     *type |= bits;
 
     CHECKOUT(largest_acked_len);
-    tmp_packno = maxno;
 #if __BYTE_ORDER == __LITTLE_ENDIAN
     tmp_packno = bswap_64(maxno);
+#else
+    tmp_packno = maxno;
 #endif
     memcpy(p, (unsigned char *) &tmp_packno + 8 - largest_acked_len,
                                                             largest_acked_len);
@@ -990,7 +984,55 @@ gquic_be_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
 }
 
 
-const struct parse_funcs lsquic_parse_funcs_gquic_Q039 =
+static int
+gquic_be_gen_crypto_frame (unsigned char *buf, size_t buf_len,
+        uint64_t offset, size_t size, gcf_read_f gcf_read, void *stream)
+{
+    assert(0);
+    return -1;
+}
+
+
+static int
+gquic_be_parse_crypto_frame (const unsigned char *buf, size_t rem_packet_sz,
+                                            struct stream_frame *stream_frame)
+{
+    assert(0);
+    return -1;
+}
+
+
+static void
+gquic_be_packno_info (const struct lsquic_conn *lconn,
+        const struct lsquic_packet_out *packet_out, unsigned *packno_off,
+        unsigned *packno_len)
+{
+    assert(0);
+}
+
+
+static unsigned
+gquic_Q043_handshake_done_frame_size (void)
+{
+    return 0;
+}
+
+
+static int
+gquic_Q043_gen_handshake_done_frame (unsigned char *buf, size_t buf_len)
+{
+    return -1;
+}
+
+
+static int
+gquic_Q043_parse_handshake_done_frame (const unsigned char *buf, size_t buf_len)
+{
+    return -1;
+}
+
+
+const struct parse_funcs lsquic_parse_funcs_gquic_Q043 =
 {
     .pf_gen_reg_pkt_header            =  gquic_be_gen_reg_pkt_header,
     .pf_parse_packet_in_finish        =  gquic_be_parse_packet_in_finish,
@@ -1009,6 +1051,7 @@ const struct parse_funcs lsquic_parse_funcs_gquic_Q039 =
     .pf_rst_frame_size                =  gquic_be_rst_frame_size,
     .pf_gen_rst_frame                 =  gquic_be_gen_rst_frame,
     .pf_parse_rst_frame               =  gquic_be_parse_rst_frame,
+    .pf_connect_close_frame_size      =  gquic_be_connect_close_frame_size,
     .pf_gen_connect_close_frame       =  gquic_be_gen_connect_close_frame,
     .pf_parse_connect_close_frame     =  gquic_be_parse_connect_close_frame,
     .pf_gen_goaway_frame              =  gquic_be_gen_goaway_frame,
@@ -1019,10 +1062,16 @@ const struct parse_funcs lsquic_parse_funcs_gquic_Q039 =
     .pf_read_float_time16             =  gquic_be_read_float_time16,
 #endif
     .pf_generate_simple_prst          =  lsquic_generate_gquic_reset,
-    .pf_parse_frame_type              =  parse_frame_type_gquic_Q035_thru_Q039,
-    .pf_turn_on_fin                   =  lsquic_turn_on_fin_Q035_thru_Q039,
+    .pf_parse_frame_type              =  lsquic_parse_frame_type_gquic_Q035_thru_Q046,
+    .pf_turn_on_fin                   =  lsquic_turn_on_fin_Q035_thru_Q046,
     .pf_packout_size                  =  lsquic_gquic_packout_size,
     .pf_packout_max_header_size       =  lsquic_gquic_packout_header_size,
     .pf_calc_packno_bits              =  lsquic_gquic_calc_packno_bits,
     .pf_packno_bits2len               =  lsquic_gquic_packno_bits2len,
+    .pf_gen_crypto_frame              =  gquic_be_gen_crypto_frame,
+    .pf_parse_crypto_frame            =  gquic_be_parse_crypto_frame,
+    .pf_packno_info                   =  gquic_be_packno_info,
+    .pf_gen_handshake_done_frame      =  gquic_Q043_gen_handshake_done_frame,
+    .pf_parse_handshake_done_frame    =  gquic_Q043_parse_handshake_done_frame,
+    .pf_handshake_done_frame_size     =  gquic_Q043_handshake_done_frame_size,
 };
