@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_parse_ietf_v1.c -- Parsing functions specific to IETF QUIC v1
  */
@@ -7,7 +7,6 @@
 #include <inttypes.h>
 #include <errno.h>
 #include <stddef.h>
-#include <stdlib.h>
 #include <string.h>
 #include <sys/queue.h>
 #ifndef WIN32
@@ -35,10 +34,11 @@
 #include "lsquic_version.h"
 #include "lsquic.h"
 #include "lsquic_byteswap.h"
-#include "lsquic_varint.h"
 #include "lsquic_conn.h"
 #include "lsquic_enc_sess.h"
 #include "lsquic_trans_params.h"
+#include "lsquic_parse_ietf.h"
+#include "lsquic_qtags.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_PARSE
 #include "lsquic_logger.h"
@@ -46,6 +46,9 @@
 #define CHECK_SPACE(need, pstart, pend)  \
     do { if ((intptr_t) (need) > ((pend) - (pstart))) { return -1; } } while (0)
 
+#define FRAME_TYPE_ACK_FREQUENCY    0xAF
+#define FRAME_TYPE_TIMESTAMP        0x2F5
+
 static int
 ietf_v1_gen_one_varint (unsigned char *, size_t, unsigned char, uint64_t);
 
@@ -171,10 +174,13 @@ write_packno (unsigned char *p, lsquic_packno_t packno,
     {
     case IQUIC_PACKNO_LEN_4:
         *p++ = packno >> 24;
+        /* fall through */
     case IQUIC_PACKNO_LEN_3:
         *p++ = packno >> 16;
+        /* fall through */
     case IQUIC_PACKNO_LEN_2:
         *p++ = packno >> 8;
+        /* fall through */
     default:
         *p++ = packno;
     }
@@ -236,8 +242,7 @@ gen_long_pkt_header (const struct lsquic_conn *lconn,
     vint_write(p, payload_len, bits, 1 << bits);
     p += 1 << bits;
 
-    p += write_packno(p, packet_out->po_packno,
-                                    lsquic_packet_out_packno_bits(packet_out));
+    p += write_packno(p, packet_out->po_packno, packno_bits);
 
     return p - buf;
 }
@@ -261,7 +266,9 @@ gen_short_pkt_header (const struct lsquic_conn *lconn,
 
     buf[0] = 0x40
            | (lsquic_packet_out_spin_bit(packet_out) << 5)
-           | (lsquic_packet_out_key_phase(packet_out) << 2)
+           | (lsquic_packet_out_square_bit(packet_out) << 4)
+           | (lsquic_packet_out_loss_bit(packet_out) << 3)
+           | (lsquic_packet_out_kp(packet_out) << 2)
            | packno_bits
            ;
 
@@ -430,9 +437,10 @@ ietf_v1_gen_stream_frame (unsigned char *buf, size_t buf_len,
 }
 
 
-static int
-ietf_v1_gen_crypto_frame (unsigned char *buf, size_t buf_len,
-        uint64_t offset, size_t size, gcf_read_f gcf_read, void *stream)
+int
+lsquic_ietf_v1_gen_crypto_frame (unsigned char *buf, unsigned char first_byte,
+            size_t buf_len, uint64_t offset, size_t size, gcf_read_f gcf_read,
+            void *stream)
 {
     unsigned char *const end = buf + buf_len;
     unsigned char *p;
@@ -453,7 +461,7 @@ ietf_v1_gen_crypto_frame (unsigned char *buf, size_t buf_len,
         size = n_avail;
 
     p = buf;
-    *p++ = 0x06;
+    *p++ = first_byte;
 
     vint_write(p, offset, obits, olen);
     p += olen;
@@ -469,6 +477,15 @@ ietf_v1_gen_crypto_frame (unsigned char *buf, size_t buf_len,
 }
 
 
+static int
+ietf_v1_gen_crypto_frame (unsigned char *buf, size_t buf_len,
+        uint64_t offset, size_t size, gcf_read_f gcf_read, void *stream)
+{
+    return lsquic_ietf_v1_gen_crypto_frame(buf, 0x6, buf_len, offset,
+                                                size, gcf_read, stream);
+}
+
+
 /* return parsed (used) buffer length */
 static int
 ietf_v1_parse_stream_frame (const unsigned char *buf, size_t rem_packet_sz,
@@ -514,6 +531,10 @@ ietf_v1_parse_stream_frame (const unsigned char *buf, size_t rem_packet_sz,
     else
         data_sz = pend - p;
 
+    /* Largest offset cannot exceed this value and we MUST detect this error */
+    if (VINT_MAX_VALUE - offset < data_sz)
+        return -1;
+
     stream_frame->stream_id             = stream_id;
     stream_frame->data_frame.df_fin     = type & 0x1;
     stream_frame->data_frame.df_offset  = offset;
@@ -528,46 +549,63 @@ ietf_v1_parse_stream_frame (const unsigned char *buf, size_t rem_packet_sz,
 }
 
 
+int
+lsquic_ietf_v1_parse_crypto_frame (const unsigned char *buf,
+                    size_t rem_packet_sz, struct stream_frame *stream_frame)
+{
+    const unsigned char *const pend = buf + rem_packet_sz;
+    const unsigned char *p = buf;
+    uint64_t offset, data_sz;
+    int r;
+
+    CHECK_SPACE(1, p, pend);
+
+    ++p;
+
+    r = vint_read(p, pend, &offset);
+    if (r < 0)
+        return -1;
+    p += r;
+
+    r = vint_read(p, pend, &data_sz);
+    if (r < 0)
+        return -1;
+    p += r;
+    CHECK_SPACE(data_sz, p, pend);
+
+    /* Largest offset cannot exceed this value and we MUST detect this error */
+    if (VINT_MAX_VALUE - offset < data_sz)
+        return -1;
+
+    stream_frame->stream_id             = ~0ULL;    /* Unset */
+    stream_frame->data_frame.df_fin     = 0;
+    stream_frame->data_frame.df_offset  = offset;
+    stream_frame->data_frame.df_size    = data_sz;
+    stream_frame->data_frame.df_data    = p;
+    stream_frame->data_frame.df_read_off= 0;
+    stream_frame->packet_in             = NULL;
+
+    assert(p <= pend);
+
+    return p + data_sz - (unsigned char *) buf;
+}
+
+
 static int
 ietf_v1_parse_crypto_frame (const unsigned char *buf, size_t rem_packet_sz,
                                         struct stream_frame *stream_frame)
 {
-    const unsigned char *const pend = buf + rem_packet_sz;
-    const unsigned char *p = buf;
-    uint64_t offset, data_sz;
-    int r;
-
-    CHECK_SPACE(1, p, pend);
-
-    assert(0x06 == *p);
-    ++p;
-
-    r = vint_read(p, pend, &offset);
-    if (r < 0)
+    if (rem_packet_sz > 0)
+    {
+        assert(0x06 == buf[0]);
+        return lsquic_ietf_v1_parse_crypto_frame(buf, rem_packet_sz,
+                                                            stream_frame);
+    }
+    else
         return -1;
-    p += r;
-
-    r = vint_read(p, pend, &data_sz);
-    if (r < 0)
-        return -1;
-    p += r;
-    CHECK_SPACE(data_sz, p, pend);
-
-    stream_frame->stream_id             = ~0ULL;    /* Unset */
-    stream_frame->data_frame.df_fin     = 0;
-    stream_frame->data_frame.df_offset  = offset;
-    stream_frame->data_frame.df_size    = data_sz;
-    stream_frame->data_frame.df_data    = p;
-    stream_frame->data_frame.df_read_off= 0;
-    stream_frame->packet_in             = NULL;
-
-    assert(p <= pend);
-
-    return p + data_sz - (unsigned char *) buf;
 }
 
 
-
 #if __GNUC__
 #   define UNLIKELY(cond) __builtin_expect(cond, 0)
 #else
@@ -656,12 +694,6 @@ ietf_v1_parse_ack_frame (const unsigned char *const buf, size_t buf_len,
         ack->flags |= AI_ECN;
     }
 
-#if LSQUIC_PARSE_ACK_TIMESTAMPS
-#error Pasing ACK timestamps not supported
-#else
-    ack->n_timestamps = 0;
-#endif
-
     return p - buf;
 }
 
@@ -818,6 +850,19 @@ ietf_v1_gen_ping_frame (unsigned char *buf, int buf_len)
 }
 
 
+static size_t
+ietf_v1_connect_close_frame_size (int app_error, unsigned error_code,
+                                unsigned frame_type, size_t reason_len)
+{
+    return 1                                                         /* Type */
+         + (1 << vint_val2bits(error_code))                    /* Error code */
+         + (app_error ? 0 : 1 << vint_val2bits(frame_type))    /* Frame type */
+         + (1 << vint_val2bits(reason_len))          /* Reason Phrase Length */
+         + reason_len
+         ;
+}
+
+
 static int
 ietf_v1_gen_connect_close_frame (unsigned char *buf, size_t buf_len,
     int app_error, unsigned error_code, const char *reason, int reason_len)
@@ -916,7 +961,7 @@ ietf_v1_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
     lsquic_packno_t packno_diff, gap, prev_low, maxno, rsize;
     size_t sz;
     const struct lsquic_packno_range *range;
-    unsigned a, b, c, addl_ack_blocks;
+    unsigned a, b, c, addl_ack_blocks, ecn_needs;
     unsigned bits[4];
     enum ecn ecn;
 
@@ -955,6 +1000,15 @@ ietf_v1_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
 
     CHECKOUT(sz);
 
+    if (ecn_counts)
+    {
+        for (ecn = 1; ecn <= 3; ++ecn)
+            bits[ecn] = vint_val2bits(ecn_counts[ecn]);
+        ecn_needs = (1 << bits[1]) + (1 << bits[2]) + (1 << bits[3]);
+    }
+    else
+        ecn_needs = 0;
+
     *p = 0x02 + !!ecn_counts;
     ++p;
 
@@ -976,13 +1030,14 @@ ietf_v1_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
         rsize = range->high - range->low;
         a = vint_val2bits(gap - 1);
         b = vint_val2bits(rsize);
+        if (ecn_needs + (1 << a) + (1 << b) > AVAIL())
+            break;
         if (addl_ack_blocks == VINT_MAX_ONE_BYTE)
         {
             memmove(block_count_p + 2, block_count_p + 1,
                                                 p - block_count_p - 1);
             ++p;
         }
-        CHECKOUT((1 << a) + (1 << b));
         vint_write(p, gap - 1, a, 1 << a);
         p += 1 << a;
         vint_write(p, rsize, b, 1 << b);
@@ -1000,9 +1055,7 @@ ietf_v1_gen_ack_frame (unsigned char *outbuf, size_t outbuf_sz,
 
     if (ecn_counts)
     {
-        for (ecn = 1; ecn <= 3; ++ecn)
-            bits[ecn] = vint_val2bits(ecn_counts[ecn]);
-        CHECKOUT((1 << bits[1]) + (1 << bits[2]) + (1 << bits[3]));
+        assert(ecn_needs <= AVAIL());
         for (ecn = 1; ecn <= 3; ++ecn)
         {
             vint_write(p, ecn_counts[ecnmap[ecn]], bits[ecnmap[ecn]], 1 << bits[ecnmap[ecn]]);
@@ -1035,20 +1088,36 @@ ietf_v1_calc_stream_frame_header_sz (lsquic_stream_id_t stream_id,
 }
 
 
+/* [draft-ietf-quic-transport-24] Section 19.6 */
 static size_t
-ietf_v1_calc_crypto_frame_header_sz (uint64_t offset)
+ietf_v1_calc_crypto_frame_header_sz (uint64_t offset, unsigned data_sz)
 {
     return 1    /* Frame type */
          + (1 << vint_val2bits(offset))
-         + 1    /* Data len */
+         + (1 << vint_val2bits(data_sz))
          ;
 }
 
 
 static enum quic_frame_type
-ietf_v1_parse_frame_type (unsigned char byte)
+ietf_v1_parse_frame_type (const unsigned char *buf, size_t len)
 {
-    return lsquic_iquic_byte2type[byte];
+    uint64_t val;
+    int s;
+
+    if (len > 0 && buf[0] < 0x40)
+        return lsquic_iquic_byte2type[buf[0]];
+
+    s = vint_read(buf, buf + len, &val);
+    if (s > 0 && (unsigned) s == (1u << vint_val2bits(val)))
+        switch (val)
+        {
+        case FRAME_TYPE_ACK_FREQUENCY:  return QUIC_FRAME_ACK_FREQUENCY;
+        case FRAME_TYPE_TIMESTAMP:      return QUIC_FRAME_TIMESTAMP;
+        default:    break;
+        }
+
+    return QUIC_FRAME_INVALID;
 }
 
 
@@ -1273,6 +1342,8 @@ ietf_v1_parse_new_conn_id (const unsigned char *buf, size_t len,
         return -1;
 
     cid_len = *p++;
+    if (cid_len == 0 || cid_len > MAX_CID_LEN)
+        return -2;
 
     if ((unsigned) (end - p) < cid_len + IQUIC_SRESET_TOKEN_SZ)
         return -1;
@@ -1349,6 +1420,76 @@ ietf_v1_parse_two_varints (const unsigned char *buf, size_t len, uint64_t *vals[
 }
 
 
+/* vals[0] is the frame type */
+static unsigned
+ietf_v1_frame_with_varints_size (unsigned n, uint64_t vals[])
+{
+    unsigned vbits, size;
+
+    assert(n > 0);
+    vbits = vint_val2bits(vals[0]);
+    size = 1 << vbits;
+    while (--n)
+    {
+        vbits = vint_val2bits(vals[n]);
+        size += 1 << vbits;
+    }
+
+    return size;
+}
+
+
+/* vals[0] is the frame type */
+static int
+ietf_v1_gen_frame_with_varints (unsigned char *buf, size_t len,
+                                    unsigned count, uint64_t vals[])
+{
+    unsigned vbits, n;
+    unsigned char *p;
+
+    if (ietf_v1_frame_with_varints_size(count, vals) > len)
+        return -1;
+
+    p = buf;
+    for (n = 0; n < count; ++n)
+    {
+        vbits = vint_val2bits(vals[n]);
+        vint_write(p, vals[n], vbits, 1 << vbits);
+        p += 1 << vbits;
+    }
+
+    return p - buf;
+}
+
+
+/* Frame type is checked when frame type is parsed.  The only use here is
+ * to calculate skip length.
+ */
+static int
+ietf_v1_parse_frame_with_varints (const unsigned char *buf, size_t len,
+            const uint64_t frame_type, unsigned count, uint64_t *vals[])
+{
+    const unsigned char *p = buf;
+    const unsigned char *const end = p + len;
+    unsigned vbits, n;
+    int s;
+
+    vbits = vint_val2bits(frame_type);
+    p += 1 << vbits;
+
+    for (n = 0; n < count; ++n)
+    {
+        s = vint_read(p, end, vals[n]);
+        if (s < 0)
+            return s;
+        p += s;
+    }
+
+    return p - buf;
+}
+
+
+
 static int
 ietf_v1_parse_stream_blocked_frame (const unsigned char *buf, size_t len,
                             lsquic_stream_id_t *stream_id, uint64_t *offset)
@@ -1554,7 +1695,7 @@ lsquic_ietf_v1_parse_packet_in_long_begin (struct lsquic_packet_in *packet_in,
     const unsigned char *const end = p + length;
     lsquic_ver_tag_t tag;
     enum header_type header_type;
-    unsigned dcil, scil, odcil;
+    unsigned dcil, scil;
     int verneg, r;
     unsigned char first_byte;
     uint64_t payload_len, token_len;
@@ -1597,7 +1738,7 @@ lsquic_ietf_v1_parse_packet_in_long_begin (struct lsquic_packet_in *packet_in,
     switch (header_type)
     {
     case HETY_INITIAL:
-        if (dcil < MIN_INITIAL_DCID_LEN)
+        if (is_server && dcil < MIN_INITIAL_DCID_LEN)
             return -1;
         r = vint_read(p, end, &token_len);
         if (r < 0)
@@ -1649,14 +1790,18 @@ lsquic_ietf_v1_parse_packet_in_long_begin (struct lsquic_packet_in *packet_in,
     case HETY_RETRY:
         if (p >= end)
             return -1;
-        odcil = *p++;
-        if (p + odcil > end || odcil > MAX_CID_LEN)
-            return -1;
-        packet_in->pi_odcid_len = odcil;
-        packet_in->pi_odcid = p - packet_in->pi_data;
-        p += odcil;
+        if (p
+            /* [draft-ietf-quic-transport-25] Section 17.2.5 says that "a
+             * client MUST discard a Retry packet with a zero-length Retry
+             * Token field."  We might as well do it here.
+             */
+            + 1
+            /* Integrity tag length: */
+            + 16 > end)
+                return -1;
         packet_in->pi_token = p - packet_in->pi_data;
-        packet_in->pi_token_size = end - p;
+        packet_in->pi_token_size = end - p - 16;
+        /* Tag validation happens later */
         p = end;
         length = end - packet_in->pi_data;
         state->pps_p      = NULL;
@@ -1694,7 +1839,7 @@ lsquic_ietf_v1_parse_packet_in_long_begin (struct lsquic_packet_in *packet_in,
 
 /* Is this a valid Initial packet?  We take the perspective of the server. */
 int
-lsquic_is_valid_ietf_v1_or_Q046_hs_packet (const unsigned char *buf,
+lsquic_is_valid_ietf_v1_or_Q046plus_hs_packet (const unsigned char *buf,
                                         size_t length, lsquic_ver_tag_t *tagp)
 {
     const unsigned char *p = buf;
@@ -1716,11 +1861,11 @@ lsquic_is_valid_ietf_v1_or_Q046_hs_packet (const unsigned char *buf,
 
     memcpy(&tag, p, 4);
     p += 4;
-    if (tag == 0)
-        return 0;       /* Client never sends version negotiation packets */
-
-    if (tag == * (uint32_t *) "Q046")
+    switch (tag)
     {
+    case 0:
+        return 0;       /* Client never sends version negotiation packets */
+    case TAG('Q', '0', '4', '6'):
         dcil = p[0] >> 4;
         if (dcil)
             dcil += 3;
@@ -1736,9 +1881,19 @@ lsquic_is_valid_ietf_v1_or_Q046_hs_packet (const unsigned char *buf,
 
         if (end - p < (ptrdiff_t) (dcil + scil + packet_len))
             return 0;
-    }
-    else
-    {
+        break;
+    case TAG('Q', '0', '5', '0'):
+        dcil = *p++;
+        if (dcil != 8)
+            return 0;
+        if (p + dcil + 1 >= end)
+            return 0;
+        p += dcil;
+        scil = *p++;
+        if (scil != 0)
+            return 0;
+        goto read_token;
+    default:
         dcil = *p++;
         if (dcil < MIN_INITIAL_DCID_LEN || dcil > MAX_CID_LEN)
             return 0;
@@ -1749,6 +1904,7 @@ lsquic_is_valid_ietf_v1_or_Q046_hs_packet (const unsigned char *buf,
         if (p + scil > end || scil > MAX_CID_LEN)
             return 0;
         p += scil;
+  read_token:
         r = vint_read(p, end, &token_len);
         if (r < 0)
             return 0;
@@ -1835,7 +1991,8 @@ popcount (unsigned v)
 
 int
 lsquic_ietf_v1_gen_ver_nego_pkt (unsigned char *buf, size_t bufsz,
-         const lsquic_cid_t *scid, const lsquic_cid_t *dcid, unsigned versions)
+         const lsquic_cid_t *scid, const lsquic_cid_t *dcid, unsigned versions,
+         uint8_t rand)
 {
     size_t need;
     int r;
@@ -1846,7 +2003,7 @@ lsquic_ietf_v1_gen_ver_nego_pkt (unsigned char *buf, size_t bufsz,
     if (need > bufsz)
         return -1;
 
-    *buf++ = 0x80 | 0x40 | rand();
+    *buf++ = 0x80 | 0x40 | rand;
     memset(buf, 0, 4);
     buf += 4;
 
@@ -1879,6 +2036,79 @@ lsquic_ietf_v1_gen_ver_nego_pkt (unsigned char *buf, size_t bufsz,
 }
 
 
+static int
+ietf_v1_gen_handshake_done_frame (unsigned char *buf, size_t buf_len)
+{
+    if (buf_len > 0)
+    {
+        *buf = 0x1E;
+        return 1;
+    }
+    else
+        return -1;
+}
+
+
+static int
+ietf_v1_parse_handshake_done_frame (const unsigned char *buf, size_t buf_len)
+{
+    assert(buf[0] == 0x1E);
+    assert(buf_len > 0);
+    return 1;
+}
+
+
+static int
+ietf_v1_gen_ack_frequency_frame (unsigned char *buf, size_t buf_len,
+    uint64_t seqno, uint64_t pack_tol, uint64_t upd_mad)
+{
+    return ietf_v1_gen_frame_with_varints(buf, buf_len, 4,
+        (uint64_t[]){ FRAME_TYPE_ACK_FREQUENCY, seqno, pack_tol, upd_mad });
+}
+
+
+static int
+ietf_v1_parse_ack_frequency_frame (const unsigned char *buf, size_t buf_len,
+    uint64_t *seqno, uint64_t *pack_tol, uint64_t *upd_mad)
+{
+    return ietf_v1_parse_frame_with_varints(buf, buf_len,
+                FRAME_TYPE_ACK_FREQUENCY,
+                3, (uint64_t *[]) { seqno, pack_tol, upd_mad });
+}
+
+
+static unsigned
+ietf_v1_ack_frequency_frame_size (uint64_t seqno, uint64_t pack_tol,
+    uint64_t upd_mad)
+{
+    return ietf_v1_frame_with_varints_size(4,
+            (uint64_t[]){ FRAME_TYPE_ACK_FREQUENCY, seqno, pack_tol, upd_mad });
+}
+
+
+static unsigned
+ietf_v1_handshake_done_frame_size (void)
+{
+    return 1;
+}
+
+
+static int
+ietf_v1_gen_timestamp_frame (unsigned char *buf, size_t buf_len,
+                                                            uint64_t timestamp)
+{
+    return ietf_v1_gen_frame_with_varints(buf, buf_len, 2,
+                            (uint64_t[]){ FRAME_TYPE_TIMESTAMP, timestamp });
+}
+
+
+static int
+ietf_v1_parse_timestamp_frame (const unsigned char *buf, size_t buf_len,
+                                                            uint64_t *timestamp)
+{
+    return ietf_v1_parse_frame_with_varints(buf, buf_len,
+                FRAME_TYPE_TIMESTAMP, 1, (uint64_t *[]) { timestamp });
+}
 
 
 const struct parse_funcs lsquic_parse_funcs_ietf_v1 =
@@ -1896,6 +2126,7 @@ const struct parse_funcs lsquic_parse_funcs_ietf_v1 =
     .pf_rst_frame_size                =  ietf_v1_rst_frame_size,
     .pf_gen_rst_frame                 =  ietf_v1_gen_rst_frame,
     .pf_parse_rst_frame               =  ietf_v1_parse_rst_frame,
+    .pf_connect_close_frame_size      =  ietf_v1_connect_close_frame_size,
     .pf_gen_connect_close_frame       =  ietf_v1_gen_connect_close_frame,
     .pf_parse_connect_close_frame     =  ietf_v1_parse_connect_close_frame,
     .pf_gen_ping_frame                =  ietf_v1_gen_ping_frame,
@@ -1942,4 +2173,12 @@ const struct parse_funcs lsquic_parse_funcs_ietf_v1 =
     .pf_gen_max_streams_frame         =  ietf_v1_gen_max_streams_frame,
     .pf_parse_max_streams_frame       =  ietf_v1_parse_max_streams_frame,
     .pf_max_streams_frame_size        =  ietf_v1_max_streams_frame_size,
+    .pf_gen_handshake_done_frame      =  ietf_v1_gen_handshake_done_frame,
+    .pf_parse_handshake_done_frame    =  ietf_v1_parse_handshake_done_frame,
+    .pf_handshake_done_frame_size     =  ietf_v1_handshake_done_frame_size,
+    .pf_gen_ack_frequency_frame       =  ietf_v1_gen_ack_frequency_frame,
+    .pf_parse_ack_frequency_frame     =  ietf_v1_parse_ack_frequency_frame,
+    .pf_ack_frequency_frame_size      =  ietf_v1_ack_frequency_frame_size,
+    .pf_gen_timestamp_frame           =  ietf_v1_gen_timestamp_frame,
+    .pf_parse_timestamp_frame         =  ietf_v1_parse_timestamp_frame,
 };
