@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_pr_queue.c -- packet request queue.
  */
@@ -35,6 +35,7 @@
 #include "lsquic_sizes.h"
 #include "lsquic_handshake.h"
 #include "lsquic_xxhash.h"
+#include "lsquic_crand.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_PRQ
 #include "lsquic_logger.h"
@@ -68,6 +69,9 @@ struct evanescent_conn
     struct pr_queue            *evc_queue;
     struct lsquic_packet_out    evc_packet_out;
     struct conn_cid_elem        evc_cces[1];
+    enum {
+        EVC_DROP    = 1 << 0,
+    }                           evc_flags;
     unsigned char               evc_buf[0];
 };
 
@@ -165,7 +169,7 @@ prq_create (unsigned max_elems, unsigned max_conns,
     }
 
     const lsquic_cid_t cid = { .len = 8, };
-    pf = select_pf_by_ver(LSQVER_039);
+    pf = select_pf_by_ver(LSQVER_043);
     len = lsquic_gquic_gen_ver_nego_pkt(prq->prq_verneg_g_buf,
                     sizeof(prq->prq_verneg_g_buf), &cid,
                     enpub->enp_settings.es_versions);
@@ -256,47 +260,33 @@ put_req (struct pr_queue *prq, struct packet_req *req)
 
 
 int
-prq_new_req (struct pr_queue *prq, enum packet_req_type type,
-         const struct lsquic_packet_in *packet_in, void *peer_ctx,
-         const struct sockaddr *local_addr, const struct sockaddr *peer_addr)
+lsquic_prq_new_req (struct pr_queue *prq, enum packet_req_type type,
+    unsigned flags, enum lsquic_version version, unsigned short data_sz,
+    const lsquic_cid_t *dcid, const lsquic_cid_t *scid, void *peer_ctx,
+    const struct sockaddr *local_addr, const struct sockaddr *peer_addr)
 {
     struct packet_req *req;
-    lsquic_ver_tag_t ver_tag;
-    enum lsquic_version version;
-    enum pr_flags flags;
-    unsigned max, size;
-
-    if (packet_in->pi_flags & PI_GQUIC)
-        flags = PR_GQUIC;
-    else
-        flags = 0;
-
-    if (packet_in->pi_quic_ver)
-    {
-        memcpy(&ver_tag, packet_in->pi_data + packet_in->pi_quic_ver,
-                                                            sizeof(ver_tag));
-        version = lsquic_tag2ver(ver_tag);
-    }
-    else /* Got to set it to something sensible... */
-        version = LSQVER_ID23;
+    unsigned max, size, rand;
 
     if (type == PACKET_REQ_PUBRES && !(flags & PR_GQUIC))
     {
-        if (packet_in->pi_data_sz <= IQUIC_MIN_SRST_SIZE)
+        if (data_sz <= IQUIC_MIN_SRST_SIZE)
         {
             LSQ_DEBUGC("not scheduling public reset: incoming packet for CID "
-                "%"CID_FMT" too small: %hu bytes",
-                CID_BITS(&packet_in->pi_dcid), packet_in->pi_data_sz);
+                "%"CID_FMT" too small: %hu bytes", CID_BITS(dcid), data_sz);
             return -1;
         }
         /* Use a random stateless reset size */
-        max = MIN(IQUIC_MAX_SRST_SIZE, packet_in->pi_data_sz - 1u);
+        max = MIN(IQUIC_MAX_SRST_SIZE, data_sz - 1u);
         if (max > IQUIC_MIN_SRST_SIZE)
-            size = IQUIC_MIN_SRST_SIZE + rand() % (max - IQUIC_MIN_SRST_SIZE);
+        {
+            rand = lsquic_crand_get_byte(prq->prq_enpub->enp_crand);
+            size = IQUIC_MIN_SRST_SIZE + rand % (max - IQUIC_MIN_SRST_SIZE);
+        }
         else
             size = IQUIC_MIN_SRST_SIZE;
         LSQ_DEBUGC("selected %u-byte reset size for CID %"CID_FMT
-            " (range is [%u, %u])", size, CID_BITS(&packet_in->pi_dcid),
+            " (range is [%u, %u])", size, CID_BITS(dcid),
             IQUIC_MIN_SRST_SIZE, max);
     }
     else
@@ -310,8 +300,8 @@ prq_new_req (struct pr_queue *prq, enum packet_req_type type,
     }
 
     req->pr_type     = type;
-    req->pr_dcid     = packet_in->pi_dcid;
-    if (lsquic_hash_find(prq->prq_reqs_hash, req, sizeof(req)))
+    req->pr_dcid     = *dcid;
+    if (lsquic_hash_find(prq->prq_reqs_hash, req, sizeof(*req)))
     {
         LSQ_DEBUG("request for this DCID and type already exists");
         put_req(prq, req);
@@ -319,7 +309,7 @@ prq_new_req (struct pr_queue *prq, enum packet_req_type type,
     }
 
     req->pr_hash_el.qhe_flags = 0;
-    if (!lsquic_hash_insert(prq->prq_reqs_hash, req, sizeof(req),
+    if (!lsquic_hash_insert(prq->prq_reqs_hash, req, sizeof(*req),
                                                     req, &req->pr_hash_el))
     {
         LSQ_DEBUG("could not insert req into hash");
@@ -330,9 +320,9 @@ prq_new_req (struct pr_queue *prq, enum packet_req_type type,
     req->pr_flags    = flags;
     req->pr_rst_sz   = size;
     req->pr_version  = version;
-    lsquic_scid_from_packet_in(packet_in, &req->pr_scid);
+    req->pr_scid     = *scid;
     req->pr_path.np_peer_ctx = peer_ctx;
-    memcpy(NP_LOCAL_SA(&req->pr_path), local_addr,
+    memcpy(req->pr_path.np_local_addr, local_addr,
                                             sizeof(req->pr_path.np_local_addr));
     memcpy(NP_PEER_SA(&req->pr_path), peer_addr,
                                             sizeof(req->pr_path.np_peer_addr));
@@ -343,6 +333,36 @@ prq_new_req (struct pr_queue *prq, enum packet_req_type type,
 }
 
 
+int
+prq_new_req (struct pr_queue *prq, enum packet_req_type type,
+         const struct lsquic_packet_in *packet_in, void *peer_ctx,
+         const struct sockaddr *local_addr, const struct sockaddr *peer_addr)
+{
+    lsquic_ver_tag_t ver_tag;
+    enum lsquic_version version;
+    enum pr_flags flags;
+    lsquic_cid_t scid;
+
+    if (packet_in->pi_flags & PI_GQUIC)
+        flags = PR_GQUIC;
+    else
+        flags = 0;
+
+    if (packet_in->pi_quic_ver)
+    {
+        memcpy(&ver_tag, packet_in->pi_data + packet_in->pi_quic_ver,
+                                                            sizeof(ver_tag));
+        version = lsquic_tag2ver(ver_tag);
+    }
+    else /* Got to set it to something sensible... */
+        version = LSQVER_ID25;
+
+    lsquic_scid_from_packet_in(packet_in, &scid);
+    return lsquic_prq_new_req(prq, type, flags, version, packet_in->pi_data_sz,
+                &packet_in->pi_dcid, &scid, peer_ctx, local_addr, peer_addr);
+}
+
+
 static size_t
 max_bufsz (const struct pr_queue *prq)
 {
@@ -371,7 +391,9 @@ get_evconn (struct pr_queue *prq)
     if (lconn)
     {
         TAILQ_REMOVE(&prq->prq_free_conns, lconn, cn_next_pr);
-        return (struct evanescent_conn *) lconn;
+        evconn = (struct evanescent_conn *) lconn;
+        evconn->evc_flags = 0;
+        return evconn;
     }
 
     bufsz = max_bufsz(prq);
@@ -407,7 +429,7 @@ prq_next_conn (struct pr_queue *prq)
     struct packet_req *req;
     struct lsquic_packet_out *packet_out;
     int (*gen_verneg) (unsigned char *, size_t, const lsquic_cid_t *,
-                                            const lsquic_cid_t *, unsigned);
+                                    const lsquic_cid_t *, unsigned, uint8_t);
     int len;
 
     lconn = TAILQ_FIRST(&prq->prq_returned_conns);
@@ -451,7 +473,8 @@ prq_next_conn (struct pr_queue *prq)
             gen_verneg = lsquic_ietf_v1_gen_ver_nego_pkt;
         len = gen_verneg(packet_out->po_data, max_bufsz(prq),
                     /* Flip SCID/DCID here: */ &req->pr_dcid, &req->pr_scid,
-                    prq->prq_enpub->enp_settings.es_versions);
+                    prq->prq_enpub->enp_settings.es_versions,
+                    lsquic_crand_get_byte(prq->prq_enpub->enp_crand));
         if (len > 0)
             packet_out->po_data_sz = len;
         else
@@ -496,6 +519,17 @@ evanescent_conn_ci_next_packet_to_send (struct lsquic_conn *lconn, size_t size)
 }
 
 
+static void
+prq_free_conn (struct pr_queue *prq, struct lsquic_conn *lconn)
+{
+    struct evanescent_conn *const evconn = (struct evanescent_conn *) lconn;
+
+    TAILQ_INSERT_HEAD(&prq->prq_free_conns, lconn, cn_next_pr);
+    put_req(prq, evconn->evc_req);
+    --prq->prq_nconns;
+}
+
+
 static void
 evanescent_conn_ci_packet_sent (struct lsquic_conn *lconn,
                             struct lsquic_packet_out *packet_out)
@@ -509,9 +543,7 @@ evanescent_conn_ci_packet_sent (struct lsquic_conn *lconn,
     LSQ_DEBUGC("sent %s packet for connection %"CID_FMT"; free resources",
         lsquic_preqt2str[ evconn->evc_req->pr_type ],
         CID_BITS(&evconn->evc_req->pr_dcid));
-    TAILQ_INSERT_HEAD(&prq->prq_free_conns, lconn, cn_next_pr);
-    put_req(prq, evconn->evc_req);
-    --prq->prq_nconns;
+    prq_free_conn(prq, lconn);
 }
 
 
@@ -525,8 +557,17 @@ evanescent_conn_ci_packet_not_sent (struct lsquic_conn *lconn,
     assert(packet_out == &evconn->evc_packet_out);
     assert(prq->prq_nconns > 0);
 
-    LSQ_DEBUG("packet not sent; put connection onto used list");
-    TAILQ_INSERT_HEAD(&prq->prq_returned_conns, lconn, cn_next_pr);
+    if (evconn->evc_flags & EVC_DROP)
+    {
+        LSQ_DEBUGC("packet not sent; drop connection %"CID_FMT,
+                                        CID_BITS(&evconn->evc_req->pr_dcid));
+        prq_free_conn(prq, lconn);
+    }
+    else
+    {
+        LSQ_DEBUG("packet not sent; put connection onto used list");
+        TAILQ_INSERT_HEAD(&prq->prq_returned_conns, lconn, cn_next_pr);
+    }
 }
 
 
@@ -615,3 +656,14 @@ const char *const lsquic_preqt2str[] =
     [PACKET_REQ_VERNEG] = "version negotiation",
     [PACKET_REQ_PUBRES] = "stateless reset",
 };
+
+
+void
+lsquic_prq_drop (struct lsquic_conn *lconn)
+{
+    struct evanescent_conn *const evconn = (void *) lconn;
+
+    evconn->evc_flags |= EVC_DROP;
+    LSQ_DEBUGC("mark for connection %"CID_FMT" for dropping",
+                                        CID_BITS(&evconn->evc_req->pr_dcid));
+}
