@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_qdec_hdl.c -- QPACK decoder streams handler
  */
@@ -25,7 +25,6 @@
 #include "lsquic_mm.h"
 #include "lsquic_engine_public.h"
 #include "lsquic_headers.h"
-#include "lsquic_http1x_if.h"
 #include "lsquic_conn.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_QDEC_HDL
@@ -139,7 +138,7 @@ lsquic_qdh_init (struct qpack_dec_hdl *qdh, struct lsquic_conn *conn,
     {
         qdh->qdh_h1x_ctor_ctx = (struct http1x_ctor_ctx) {
             .conn           = conn,
-            .max_headers_sz = 0x10000,  /* XXX */
+            .max_headers_sz = MAX_HTTP1X_HEADERS_SIZE,
             .is_server      = is_server,
         };
         qdh->qdh_hsi_ctx = &qdh->qdh_h1x_ctor_ctx;
@@ -223,7 +222,16 @@ qdh_out_on_write (struct lsquic_stream *stream, lsquic_stream_ctx_t *ctx)
         LSQ_DEBUG("wrote %zd bytes to stream", nw);
         (void) lsquic_stream_flush(stream);
         if (lsquic_frab_list_empty(&qdh->qdh_fral))
+        {
             lsquic_stream_wantwrite(stream, 0);
+            if (qdh->qdh_on_dec_sent_func)
+            {
+                LSQ_DEBUG("buffered data written: call callback");
+                qdh->qdh_on_dec_sent_func(qdh->qdh_on_dec_sent_ctx);
+                qdh->qdh_on_dec_sent_func = NULL;
+                qdh->qdh_on_dec_sent_ctx = NULL;
+            }
+        }
     }
     else
     {
@@ -294,7 +302,7 @@ qdh_read_encoder_stream (void *ctx, const unsigned char *buf, size_t sz,
     s = lsqpack_dec_enc_in(&qdh->qdh_decoder, buf, sz);
     if (s != 0)
     {
-        LSQ_INFO("error reading decoder stream");
+        LSQ_INFO("error reading encoder stream");
         qerr = lsqpack_dec_get_err_info(&qdh->qdh_decoder);
         qdh->qdh_conn->cn_if->ci_abort_error(qdh->qdh_conn, 1,
             HEC_QPACK_DECODER_STREAM_ERROR, "Error interpreting QPACK encoder "
@@ -376,18 +384,79 @@ qdh_hblock_unblocked (void *stream_p)
 }
 
 
+struct cont_len
+{
+    unsigned long long      value;
+    int                     has;    /* 1: set, 0: not set, -1: invalid */
+};
+
+
+static void
+process_content_length (const struct qpack_dec_hdl *qdh /* for logging */,
+            struct cont_len *cl, const char *val /* not NUL-terminated */,
+                                                                unsigned len)
+{
+    char *endcl, cont_len_buf[30];
+
+    if (0 == cl->has)
+    {
+        if (len >= sizeof(cont_len_buf))
+        {
+            LSQ_DEBUG("content-length has invalid value `%.*s'",
+                                                            (int) len, val);
+            cl->has = -1;
+            return;
+        }
+        memcpy(cont_len_buf, val, len);
+        cont_len_buf[len] = '\0';
+        cl->value = strtoull(cont_len_buf, &endcl, 10);
+        if (*endcl == '\0' && !(ULLONG_MAX == cl->value && ERANGE == errno))
+        {
+            cl->has = 1;
+            LSQ_DEBUG("content length is %llu", cl->value);
+        }
+        else
+        {
+            cl->has = -1;
+            LSQ_DEBUG("content-length has invalid value `%.*s'",
+                (int) len, val);
+        }
+    }
+    else if (cl->has > 0)
+    {
+        LSQ_DEBUG("header set has two content-length: ambiguous, "
+            "turn off checking");
+        cl->has = -1;
+    }
+}
+
+
+static int
+is_content_length (const struct lsqpack_header *header)
+{
+    return ((header->qh_flags & QH_ID_SET) && header->qh_static_id == 4)
+        || (header->qh_name_len == 14 && header->qh_name[0] == 'c'
+                    && 0 == memcmp(header->qh_name + 1, "ontent-length", 13))
+        ;
+}
+
+
 static int
 qdh_supply_hset_to_stream (struct qpack_dec_hdl *qdh,
             struct lsquic_stream *stream, struct lsqpack_header_list *qlist)
 {
     const struct lsquic_hset_if *const hset_if = qdh->qdh_enpub->enp_hsi_if;
+    const unsigned hpack_static_table_size = 61;
     struct uncompressed_headers *uh = NULL;
     const struct lsqpack_header *header;
     enum lsquic_header_status st;
+    int push_promise;
     unsigned i;
     void *hset;
+    struct cont_len cl;
 
-    hset = hset_if->hsi_create_header_set(qdh->qdh_hsi_ctx, 0);
+    push_promise = lsquic_stream_header_is_pp(stream);
+    hset = hset_if->hsi_create_header_set(qdh->qdh_hsi_ctx, push_promise);
     if (!hset)
     {
         LSQ_INFO("call to hsi_create_header_set failed");
@@ -396,20 +465,29 @@ qdh_supply_hset_to_stream (struct qpack_dec_hdl *qdh,
 
     LSQ_DEBUG("got header set for stream %"PRIu64, stream->id);
 
+    cl.has = 0;
     for (i = 0; i < qlist->qhl_count; ++i)
     {
         header = qlist->qhl_headers[i];
         LSQ_DEBUG("%.*s: %.*s", header->qh_name_len, header->qh_name,
                                         header->qh_value_len, header->qh_value);
         st = hset_if->hsi_process_header(hset,
-                    header->qh_flags & QH_ID_SET ? 62 /* XXX: 62 */ + header->qh_static_id : 0,
+                    header->qh_flags & QH_ID_SET ?
+                        hpack_static_table_size + 1 + header->qh_static_id : 0,
                     header->qh_name, header->qh_name_len,
                     header->qh_value, header->qh_value_len);
         if (st != LSQUIC_HDR_OK)
+        {
+            LSQ_INFO("header process returned non-OK code %u", (unsigned) st);
             goto err;
+        }
+        if (is_content_length(header))
+            process_content_length(qdh, &cl, header->qh_value,
+                                                        header->qh_value_len);
     }
 
     lsqpack_dec_destroy_header_list(qlist);
+    qlist = NULL;
     st = hset_if->hsi_process_header(hset, 0, 0, 0, 0, 0);
     if (st != LSQUIC_HDR_OK)
         goto err;
@@ -428,16 +506,35 @@ qdh_supply_hset_to_stream (struct qpack_dec_hdl *qdh,
         goto err;
     LSQ_DEBUG("converted qlist to hset and gave it to stream %"PRIu64,
                                                                 stream->id);
+    if (cl.has > 0)
+        (void) lsquic_stream_verify_len(stream, cl.value);
     return 0;
 
   err:
-    lsqpack_dec_destroy_header_list(qlist);
+    if (qlist)
+        lsqpack_dec_destroy_header_list(qlist);
     hset_if->hsi_discard_header_set(hset);
     free(uh);
     return -1;
 }
 
 
+/* Releases qlist */
+static int
+qdh_process_qlist (struct qpack_dec_hdl *qdh,
+            struct lsquic_stream *stream, struct lsqpack_header_list *qlist)
+{
+    if (!lsquic_stream_header_is_trailer(stream))
+        return qdh_supply_hset_to_stream(qdh, stream, qlist);
+    else
+    {
+        LSQ_DEBUG("discard trailer header set");
+        lsqpack_dec_destroy_header_list(qlist);
+        return 0;
+    }
+}
+
+
 static enum lsqpack_read_header_status
 qdh_header_read_results (struct qpack_dec_hdl *qdh,
         struct lsquic_stream *stream, enum lsqpack_read_header_status rhs,
@@ -450,7 +547,7 @@ qdh_header_read_results (struct qpack_dec_hdl *qdh,
     {
         if (qlist)
         {
-            if (0 != qdh_supply_hset_to_stream(qdh, stream, qlist))
+            if (0 != qdh_process_qlist(qdh, stream, qlist))
                 return LQRHS_ERROR;
             if (qdh->qdh_dec_sm_out)
             {
@@ -465,7 +562,8 @@ qdh_header_read_results (struct qpack_dec_hdl *qdh,
         }
         else
         {
-            assert(0);  /* XXX TODO What do we do here? */
+            LSQ_WARN("read header status is DONE but header list is not set");
+            assert(0);
             return LQRHS_ERROR;
         }
     }
@@ -568,3 +666,24 @@ lsquic_qdh_cancel_stream (struct qpack_dec_hdl *qdh,
         lsquic_qdh_unref_stream(qdh, stream);
     }
 }
+
+
+int
+lsquic_qdh_arm_if_unsent (struct qpack_dec_hdl *qdh, void (*func)(void *),
+                                                                    void *ctx)
+{
+    size_t bytes;
+
+    /* Use size of a single frab list buffer as an arbitrary threshold */
+    bytes = lsquic_frab_list_size(&qdh->qdh_fral);
+    if (bytes <= qdh->qdh_fral.fl_buf_size)
+        return 0;
+    else
+    {
+        LSQ_DEBUG("have %zu bytes of unsent QPACK decoder stream data: set "
+            "up callback", bytes);
+        qdh->qdh_on_dec_sent_func = func;
+        qdh->qdh_on_dec_sent_ctx  = ctx;
+        return 1;
+    }
+}
