@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef LSQUIC_SEND_CTL_H
 #define LSQUIC_SEND_CTL_H 1
 
@@ -44,6 +44,10 @@ enum send_ctl_flags {
     SC_1RTT_ACKED   =  1 << 11,
     SC_APP_LIMITED  =  1 << 12,
     SC_ECN          =  1 << 13,
+    SC_QL_BITS      =  1 << 14,
+    SC_SANITY_CHECK =  1 << 15,
+    SC_CIDLEN       =  1 << 16,     /* sc_cidlen is set */
+    SC_POISON       =  1 << 17,     /* poisoned packet exists */
 };
 
 typedef struct lsquic_send_ctl {
@@ -60,6 +64,7 @@ typedef struct lsquic_send_ctl {
     lsquic_time_t                   sc_largest_acked_sent_time;
     lsquic_time_t                   sc_last_sent_time;
     lsquic_time_t                   sc_last_rto_time;
+    int                           (*sc_can_send)(struct lsquic_send_ctl *);
     unsigned                        sc_bytes_unacked_retx;
     unsigned                        sc_bytes_scheduled;
     union {
@@ -124,6 +129,10 @@ typedef struct lsquic_send_ctl {
     unsigned                        sc_retry_count;
     unsigned                        sc_rt_count;    /* Count round trips */
     lsquic_packno_t                 sc_cur_rt_end;
+    lsquic_packno_t                 sc_gap;
+    unsigned                        sc_loss_count;  /* Used to set loss bit */
+    unsigned                        sc_square_count;/* Used to set square bit */
+    signed char                     sc_cidlen;      /* For debug purposes */
 } lsquic_send_ctl_t;
 
 void
@@ -169,11 +178,16 @@ lsquic_send_ctl_expire_all (lsquic_send_ctl_t *ctl);
 #define lsquic_send_ctl_largest_ack2ed(ctl, pns) \
                                             (+(ctl)->sc_largest_ack2ed[pns])
 
-#if LSQUIC_EXTRA_CHECKS
 void
-lsquic_send_ctl_sanity_check (const lsquic_send_ctl_t *ctl);
+lsquic_send_ctl_do_sanity_check (const struct lsquic_send_ctl *ctl);
+
+#ifndef NDEBUG
+#define lsquic_send_ctl_sanity_check(ctl) do {                      \
+    if ((ctl)->sc_flags & SC_SANITY_CHECK)                          \
+        lsquic_send_ctl_do_sanity_check(ctl);                       \
+} while (0)
 #else
-#   define lsquic_send_ctl_sanity_check(ctl)
+#define lsquic_send_ctl_sanity_check(ctl)
 #endif
 
 int
@@ -281,8 +295,11 @@ lsquic_send_ctl_schedule_buffered (lsquic_send_ctl_t *, enum buf_packet_type);
     TAILQ_FIRST(&(ctl)->sc_buffered_packets[BPT_HIGHEST_PRIO].bpq_packets)  \
  || TAILQ_FIRST(&(ctl)->sc_buffered_packets[BPT_OTHER_PRIO].bpq_packets  ))
 
+#define lsquic_send_ctl_has_buffered_high(ctl) (                            \
+    !TAILQ_EMPTY(&(ctl)->sc_buffered_packets[BPT_HIGHEST_PRIO].bpq_packets))
+
 #define lsquic_send_ctl_invalidate_bpt_cache(ctl) do {      \
-    (ctl)->sc_cached_bpt.stream_id = 0;                     \
+    (ctl)->sc_cached_bpt.stream_id = UINT64_MAX;            \
 } while (0)
 
 #ifndef NDEBUG
@@ -328,17 +345,11 @@ lsquic_send_ctl_pacer_blocked (struct lsquic_send_ctl *);
 int
 lsquic_send_ctl_sched_is_blocked (struct lsquic_send_ctl *);
 
-int
-
-lsquic_send_ctl_buffered_and_same_prio_as_headers (struct lsquic_send_ctl *,
-                                                const struct lsquic_stream *);
-
 void
 lsquic_send_ctl_verneg_done (struct lsquic_send_ctl *);
 
 int
-lsquic_send_ctl_retry (struct lsquic_send_ctl *, const unsigned char *,
-                                                                size_t, int);
+lsquic_send_ctl_retry (struct lsquic_send_ctl *, const unsigned char *, size_t);
 
 int
 lsquic_send_ctl_set_token (struct lsquic_send_ctl *,
@@ -360,4 +371,20 @@ void
 lsquic_send_ctl_maybe_app_limited (struct lsquic_send_ctl *,
                                             const struct network_path *);
 
+#define lsquic_send_ctl_do_ql_bits(ctl) do {                       \
+    (ctl)->sc_flags |= SC_QL_BITS;                                 \
+} while (0)
+
+void
+lsquic_send_ctl_cidlen_change (struct lsquic_send_ctl *,
+                                unsigned orig_cid_len, unsigned new_cid_len);
+
+void
+lsquic_send_ctl_begin_optack_detection (struct lsquic_send_ctl *);
+
+#define lsquic_send_ctl_n_unacked(ctl_) ((ctl_)->sc_n_in_flight_retx)
+
+void
+lsquic_send_ctl_path_validated (struct lsquic_send_ctl *);
+
 #endif
