@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -34,7 +34,7 @@ CeHeader::CeHeader()
     , m_iPrivLen(0)
     , m_lSize(0)
     , m_inode(0)
-    , m_lastMod(0)
+    , m_tmFileLastMod(0)
 {
 }
 
