@@ -787,13 +787,11 @@ void LsShm::unmap()
 }
 
 
-LsShmOffset_t LsShm::allocPage(LsShmSize_t pagesize, int &remap)
+LsShmOffset_t LsShm::allocPage(LsShmSize_t pagesize)
 {
     LsShmOffset_t offset;
     LsShmSize_t availSize;
 
-    remap = 0;
-
     //
     //  MUTEX SHOULD BE HERE for multi process/thread environment
     // Only use lock when m_status is Ready.
