@@ -294,6 +294,10 @@ void *RadixTree::bestMatch(const char *pLabel, int iLabelLen) const
                                m_iFlags | RTFLAG_BESTMATCH);
 }
 
+const char *RadixTree::getRootLabel()
+{
+    return  m_pRoot ? &m_pRoot->label[0] : "";
+}
 
 int RadixTree::for_each(rn_foreach fun)
 {
@@ -782,13 +786,14 @@ int RadixNode::getHeader(int iFlags, ls_xpool_t *pool, const char *pLabel,
     ls_str_t hashMatch;
     int iOffset;
     int i, iExact = getNumExact();
+
     switch (getState())
     {
     case RNSTATE_NOCHILD:
         break;
     case RNSTATE_CNODE:
         pTmp = m_pCHeaders;
-        if ((pTmp->len == iLabelLen)
+        if (pTmp && (pTmp->len == iLabelLen)
             && (rnDoCmp(iFlags, &pTmp->label[0], pLabel, pTmp->len) == 0))
         {
             pHeader = pTmp;
@@ -804,7 +809,7 @@ int RadixNode::getHeader(int iFlags, ls_xpool_t *pool, const char *pLabel,
         return 1;
     case RNSTATE_PNODE:
         pTmp = m_pCHeaders;
-        if ((pTmp->len == iLabelLen)
+        if (pTmp && (pTmp->len == iLabelLen)
             && (rnDoCmp(iFlags, &pTmp->label[0], pLabel, iLabelLen) == 0))
         {
             pHeader = pTmp;
@@ -815,7 +820,7 @@ int RadixNode::getHeader(int iFlags, ls_xpool_t *pool, const char *pLabel,
         pTmp = m_pCHeaders;
         for (i = 0; i < iExact; ++i)
         {
-            if ((iLabelLen == pTmp->len)
+            if (pTmp && (iLabelLen == pTmp->len)
                 && (rnDoCmp(iFlags, &pTmp->label[0], pLabel, pTmp->len) == 0))
             {
                 pHeader = pTmp;
@@ -842,7 +847,7 @@ int RadixNode::getHeader(int iFlags, ls_xpool_t *pool, const char *pLabel,
         for (i = 0; i < iExact; ++i)
         {
             pTmp = m_pPHeaders[i];
-            if ((iLabelLen == pTmp->len)
+            if (pTmp && (iLabelLen == pTmp->len)
                 && (rnDoCmp(iFlags, &pTmp->label[0], pLabel, pTmp->len) == 0))
             {
                 pHeader = pTmp;
